<?php
/*
 * Copyright 2014 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

/**
 * Service definition for Sheets (v4).
 *
 * <p>
 * Reads and writes Google Sheets.</p>
 *
 * <p>
 * For more information about this service, see the API
 * <a href="https://developers.google.com/sheets/" target="_blank">Documentation</a>
 * </p>
 *
 * @author Google, Inc.
 */
class Google_Service_Sheets extends Google_Service
{
  /** See, edit, create, and delete all of your Google Drive files. */
  const DRIVE =
      "https://www.googleapis.com/auth/drive";
  /** View and manage Google Drive files and folders that you have opened or created with this app. */
  const DRIVE_FILE =
      "https://www.googleapis.com/auth/drive.file";
  /** See and download all your Google Drive files. */
  const DRIVE_READONLY =
      "https://www.googleapis.com/auth/drive.readonly";
  /** See, edit, create, and delete your spreadsheets in Google Drive. */
  const SPREADSHEETS =
      "https://www.googleapis.com/auth/spreadsheets";
  /** View your Google Spreadsheets. */
  const SPREADSHEETS_READONLY =
      "https://www.googleapis.com/auth/spreadsheets.readonly";

  public $spreadsheets;
  public $spreadsheets_developerMetadata;
  public $spreadsheets_sheets;
  public $spreadsheets_values;
  
  /**
   * Constructs the internal representation of the Sheets service.
   *
   * @param Google_Client $client
   */
  public function __construct(Google_Client $client)
  {
    parent::__construct($client);
    $this->rootUrl = 'https://sheets.googleapis.com/';
    $this->servicePath = '';
    $this->batchPath = 'batch';
    $this->version = 'v4';
    $this->serviceName = 'sheets';

    $this->spreadsheets = new Google_Service_Sheets_Resource_Spreadsheets(
        $this,
        $this->serviceName,
        'spreadsheets',
        array(
          'methods' => array(
            'batchUpdate' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}:batchUpdate',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
              ),
            ),'create' => array(
              'path' => 'v4/spreadsheets',
              'httpMethod' => 'POST',
              'parameters' => array(),
            ),'get' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}',
              'httpMethod' => 'GET',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'ranges' => array(
                  'location' => 'query',
                  'type' => 'string',
                  'repeated' => true,
                ),
                'includeGridData' => array(
                  'location' => 'query',
                  'type' => 'boolean',
                ),
              ),
            ),'getByDataFilter' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}:getByDataFilter',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
              ),
            ),
          )
        )
    );
    $this->spreadsheets_developerMetadata = new Google_Service_Sheets_Resource_SpreadsheetsDeveloperMetadata(
        $this,
        $this->serviceName,
        'developerMetadata',
        array(
          'methods' => array(
            'get' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/developerMetadata/{metadataId}',
              'httpMethod' => 'GET',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'metadataId' => array(
                  'location' => 'path',
                  'type' => 'integer',
                  'required' => true,
                ),
              ),
            ),'search' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/developerMetadata:search',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
              ),
            ),
          )
        )
    );

    $this->spreadsheets_values = new Google_Service_Sheets_Resource_SpreadsheetsValues(
        $this,
        $this->serviceName,
        'values',
        array(
          'methods' => array(
            'append' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values/{range}:append',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'range' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'insertDataOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'valueInputOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'responseDateTimeRenderOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'includeValuesInResponse' => array(
                  'location' => 'query',
                  'type' => 'boolean',
                ),
                'responseValueRenderOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
              ),
            ),'batchClear' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values:batchClear',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
              ),
            ),'batchClearByDataFilter' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values:batchClearByDataFilter',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
              ),
            ),'batchGet' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values:batchGet',
              'httpMethod' => 'GET',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'valueRenderOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'dateTimeRenderOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'ranges' => array(
                  'location' => 'query',
                  'type' => 'string',
                  'repeated' => true,
                ),
                'majorDimension' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
              ),
            ),'batchGetByDataFilter' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values:batchGetByDataFilter',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
              ),
            ),'batchUpdate' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values:batchUpdate',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
              ),
            ),'batchUpdateByDataFilter' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values:batchUpdateByDataFilter',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
              ),
            ),'clear' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values/{range}:clear',
              'httpMethod' => 'POST',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'range' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
              ),
            ),'get' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values/{range}',
              'httpMethod' => 'GET',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'range' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'majorDimension' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'valueRenderOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'dateTimeRenderOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
              ),
            ),'update' => array(
              'path' => 'v4/spreadsheets/{spreadsheetId}/values/{range}',
              'httpMethod' => 'PUT',
              'parameters' => array(
                'spreadsheetId' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'range' => array(
                  'location' => 'path',
                  'type' => 'string',
                  'required' => true,
                ),
                'responseValueRenderOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'valueInputOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'responseDateTimeRenderOption' => array(
                  'location' => 'query',
                  'type' => 'string',
                ),
                'includeValuesInResponse' => array(
                  'location' => 'query',
                  'type' => 'boolean',
                ),
              ),
            ),
          )
        )
    );
  }
}


class Google_Service_Sheets_Resource_Spreadsheets extends Google_Service_Resource
{
    /**
     * Applies one or more updates to the spreadsheet.
     *
     * Each request is validated before being applied. If any request is not valid
     * then the entire request will fail and nothing will be applied.
     *
     * Some requests have replies to give you some information about how they are
     * applied. The replies will mirror the requests.  For example, if you applied 4
     * updates and the 3rd one had a reply, then the response will have 2 empty
     * replies, the actual reply, and another empty reply, in that order.
     *
     * Due to the collaborative nature of spreadsheets, it is not guaranteed that
     * the spreadsheet will reflect exactly your changes after this completes,
     * however it is guaranteed that the updates in the request will be applied
     * together atomically. Your changes may be altered with respect to collaborator
     * changes. If there are no collaborators, the spreadsheet should reflect your
     * changes. (spreadsheets.batchUpdate)
     *
     * @param string $spreadsheetId The spreadsheet to apply the updates to.
     * @param Google_Service_Sheets_BatchUpdateSpreadsheetRequest $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_BatchUpdateSpreadsheetResponse
     */
    public function batchUpdate($spreadsheetId, Google_Service_Sheets_BatchUpdateSpreadsheetRequest $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('batchUpdate', array($params), "Google_Service_Sheets_BatchUpdateSpreadsheetResponse");
    }
    /**
     * Creates a spreadsheet, returning the newly created spreadsheet.
     * (spreadsheets.create)
     *
     * @param Google_Service_Sheets_Spreadsheet $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_Spreadsheet
     */
    public function create(Google_Service_Sheets_Spreadsheet $postBody, $optParams = array())
    {
        $params = array('postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('create', array($params), "Google_Service_Sheets_Spreadsheet");
    }
    /**
     * Returns the spreadsheet at the given ID. The caller must specify the
     * spreadsheet ID.
     *
     * By default, data within grids will not be returned. You can include grid data
     * one of two ways:
     *
     * * Specify a field mask listing your desired fields using the `fields` URL
     * parameter in HTTP
     *
     * * Set the includeGridData URL parameter to true.  If a field mask is set, the
     * `includeGridData` parameter is ignored
     *
     * For large spreadsheets, it is recommended to retrieve only the specific
     * fields of the spreadsheet that you want.
     *
     * To retrieve only subsets of the spreadsheet, use the ranges URL parameter.
     * Multiple ranges can be specified.  Limiting the range will return only the
     * portions of the spreadsheet that intersect the requested ranges. Ranges are
     * specified using A1 notation. (spreadsheets.get)
     *
     * @param string $spreadsheetId The spreadsheet to request.
     * @param array $optParams Optional parameters.
     *
     * @opt_param string ranges The ranges to retrieve from the spreadsheet.
     * @opt_param bool includeGridData True if grid data should be returned. This
     * parameter is ignored if a field mask was set in the request.
     * @return Google_Service_Sheets_Spreadsheet
     */
    public function get($spreadsheetId, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId);
        $params = array_merge($params, $optParams);
        return $this->call('get', array($params), "Google_Service_Sheets_Spreadsheet");
    }
    /**
     * Returns the spreadsheet at the given ID. The caller must specify the
     * spreadsheet ID.
     *
     * This method differs from GetSpreadsheet in that it allows selecting which
     * subsets of spreadsheet data to return by specifying a dataFilters parameter.
     * Multiple DataFilters can be specified.  Specifying one or more data filters
     * will return the portions of the spreadsheet that intersect ranges matched by
     * any of the filters.
     *
     * By default, data within grids will not be returned. You can include grid data
     * one of two ways:
     *
     * * Specify a field mask listing your desired fields using the `fields` URL
     * parameter in HTTP
     *
     * * Set the includeGridData parameter to true.  If a field mask is set, the
     * `includeGridData` parameter is ignored
     *
     * For large spreadsheets, it is recommended to retrieve only the specific
     * fields of the spreadsheet that you want. (spreadsheets.getByDataFilter)
     *
     * @param string $spreadsheetId The spreadsheet to request.
     * @param Google_Service_Sheets_GetSpreadsheetByDataFilterRequest $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_Spreadsheet
     */
    public function getByDataFilter($spreadsheetId, Google_Service_Sheets_GetSpreadsheetByDataFilterRequest $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('getByDataFilter', array($params), "Google_Service_Sheets_Spreadsheet");
    }
}

class Google_Service_Sheets_Resource_SpreadsheetsDeveloperMetadata extends Google_Service_Resource
{
    /**
     * Returns the developer metadata with the specified ID. The caller must specify
     * the spreadsheet ID and the developer metadata's unique metadataId.
     * (developerMetadata.get)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to retrieve metadata
     * from.
     * @param int $metadataId The ID of the developer metadata to retrieve.
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_DeveloperMetadata
     */
    public function get($spreadsheetId, $metadataId, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'metadataId' => $metadataId);
        $params = array_merge($params, $optParams);
        return $this->call('get', array($params), "Google_Service_Sheets_DeveloperMetadata");
    }
    /**
     * Returns all developer metadata matching the specified DataFilter. If the
     * provided DataFilter represents a DeveloperMetadataLookup object, this will
     * return all DeveloperMetadata entries selected by it. If the DataFilter
     * represents a location in a spreadsheet, this will return all developer
     * metadata associated with locations intersecting that region.
     * (developerMetadata.search)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to retrieve metadata
     * from.
     * @param Google_Service_Sheets_SearchDeveloperMetadataRequest $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_SearchDeveloperMetadataResponse
     */
    public function search($spreadsheetId, Google_Service_Sheets_SearchDeveloperMetadataRequest $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('search', array($params), "Google_Service_Sheets_SearchDeveloperMetadataResponse");
    }
}

class Google_Service_Sheets_Resource_SpreadsheetsValues extends Google_Service_Resource
{
    /**
     * Appends values to a spreadsheet. The input range is used to search for
     * existing data and find a "table" within that range. Values will be appended
     * to the next row of the table, starting with the first column of the table.
     * See the [guide](/sheets/api/guides/values#appending_values) and [sample
     * code](/sheets/api/samples/writing#append_values) for specific details of how
     * tables are detected and data is appended.
     *
     * The caller must specify the spreadsheet ID, range, and a valueInputOption.
     * The `valueInputOption` only controls how the input data will be added to the
     * sheet (column-wise or row-wise), it does not influence what cell the data
     * starts being written to. (values.append)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to update.
     * @param string $range The A1 notation of a range to search for a logical table
     * of data. Values will be appended after the last row of the table.
     * @param Google_Service_Sheets_ValueRange $postBody
     * @param array $optParams Optional parameters.
     *
     * @opt_param string insertDataOption How the input data should be inserted.
     * @opt_param string valueInputOption How the input data should be interpreted.
     * @opt_param string responseDateTimeRenderOption Determines how dates, times,
     * and durations in the response should be rendered. This is ignored if
     * response_value_render_option is FORMATTED_VALUE. The default dateTime render
     * option is [DateTimeRenderOption.SERIAL_NUMBER].
     * @opt_param bool includeValuesInResponse Determines if the update response
     * should include the values of the cells that were appended. By default,
     * responses do not include the updated values.
     * @opt_param string responseValueRenderOption Determines how values in the
     * response should be rendered. The default render option is
     * ValueRenderOption.FORMATTED_VALUE.
     * @return Google_Service_Sheets_AppendValuesResponse
     */
    public function append($spreadsheetId, $range, Google_Service_Sheets_ValueRange $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'range' => $range, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('append', array($params), "Google_Service_Sheets_AppendValuesResponse");
    }
    /**
     * Clears one or more ranges of values from a spreadsheet. The caller must
     * specify the spreadsheet ID and one or more ranges. Only values are cleared --
     * all other properties of the cell (such as formatting, data validation, etc..)
     * are kept. (values.batchClear)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to update.
     * @param Google_Service_Sheets_BatchClearValuesRequest $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_BatchClearValuesResponse
     */
    public function batchClear($spreadsheetId, Google_Service_Sheets_BatchClearValuesRequest $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('batchClear', array($params), "Google_Service_Sheets_BatchClearValuesResponse");
    }
    /**
     * Clears one or more ranges of values from a spreadsheet. The caller must
     * specify the spreadsheet ID and one or more DataFilters. Ranges matching any
     * of the specified data filters will be cleared.  Only values are cleared --
     * all other properties of the cell (such as formatting, data validation, etc..)
     * are kept. (values.batchClearByDataFilter)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to update.
     * @param Google_Service_Sheets_BatchClearValuesByDataFilterRequest $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_BatchClearValuesByDataFilterResponse
     */
    public function batchClearByDataFilter($spreadsheetId, Google_Service_Sheets_BatchClearValuesByDataFilterRequest $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('batchClearByDataFilter', array($params), "Google_Service_Sheets_BatchClearValuesByDataFilterResponse");
    }
    /**
     * Returns one or more ranges of values from a spreadsheet. The caller must
     * specify the spreadsheet ID and one or more ranges. (values.batchGet)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to retrieve data from.
     * @param array $optParams Optional parameters.
     *
     * @opt_param string valueRenderOption How values should be represented in the
     * output. The default render option is ValueRenderOption.FORMATTED_VALUE.
     * @opt_param string dateTimeRenderOption How dates, times, and durations should
     * be represented in the output. This is ignored if value_render_option is
     * FORMATTED_VALUE. The default dateTime render option is
     * [DateTimeRenderOption.SERIAL_NUMBER].
     * @opt_param string ranges The A1 notation of the values to retrieve.
     * @opt_param string majorDimension The major dimension that results should use.
     *
     * For example, if the spreadsheet data is: `A1=1,B1=2,A2=3,B2=4`, then
     * requesting `range=A1:B2,majorDimension=ROWS` will return `[[1,2],[3,4]]`,
     * whereas requesting `range=A1:B2,majorDimension=COLUMNS` will return
     * `[[1,3],[2,4]]`.
     * @return Google_Service_Sheets_BatchGetValuesResponse
     */
    public function batchGet($spreadsheetId, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId);
        $params = array_merge($params, $optParams);
        return $this->call('batchGet', array($params), "Google_Service_Sheets_BatchGetValuesResponse");
    }
    /**
     * Returns one or more ranges of values that match the specified data filters.
     * The caller must specify the spreadsheet ID and one or more DataFilters.
     * Ranges that match any of the data filters in the request will be returned.
     * (values.batchGetByDataFilter)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to retrieve data from.
     * @param Google_Service_Sheets_BatchGetValuesByDataFilterRequest $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_BatchGetValuesByDataFilterResponse
     */
    public function batchGetByDataFilter($spreadsheetId, Google_Service_Sheets_BatchGetValuesByDataFilterRequest $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('batchGetByDataFilter', array($params), "Google_Service_Sheets_BatchGetValuesByDataFilterResponse");
    }
    /**
     * Sets values in one or more ranges of a spreadsheet. The caller must specify
     * the spreadsheet ID, a valueInputOption, and one or more ValueRanges.
     * (values.batchUpdate)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to update.
     * @param Google_Service_Sheets_BatchUpdateValuesRequest $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_BatchUpdateValuesResponse
     */
    public function batchUpdate($spreadsheetId, Google_Service_Sheets_BatchUpdateValuesRequest $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('batchUpdate', array($params), "Google_Service_Sheets_BatchUpdateValuesResponse");
    }
    /**
     * Sets values in one or more ranges of a spreadsheet. The caller must specify
     * the spreadsheet ID, a valueInputOption, and one or more
     * DataFilterValueRanges. (values.batchUpdateByDataFilter)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to update.
     * @param Google_Service_Sheets_BatchUpdateValuesByDataFilterRequest $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_BatchUpdateValuesByDataFilterResponse
     */
    public function batchUpdateByDataFilter($spreadsheetId, Google_Service_Sheets_BatchUpdateValuesByDataFilterRequest $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('batchUpdateByDataFilter', array($params), "Google_Service_Sheets_BatchUpdateValuesByDataFilterResponse");
    }
    /**
     * Clears values from a spreadsheet. The caller must specify the spreadsheet ID
     * and range. Only values are cleared -- all other properties of the cell (such
     * as formatting, data validation, etc..) are kept. (values.clear)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to update.
     * @param string $range The A1 notation of the values to clear.
     * @param Google_Service_Sheets_ClearValuesRequest $postBody
     * @param array $optParams Optional parameters.
     * @return Google_Service_Sheets_ClearValuesResponse
     */
    public function clear($spreadsheetId, $range, Google_Service_Sheets_ClearValuesRequest $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'range' => $range, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);
        return $this->call('clear', array($params), "Google_Service_Sheets_ClearValuesResponse");
    }
    /**
     * Returns a range of values from a spreadsheet. The caller must specify the
     * spreadsheet ID and a range. (values.get)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to retrieve data from.
     * @param string $range The A1 notation of the values to retrieve.
     * @param array $optParams Optional parameters.
     *
     * @opt_param string majorDimension The major dimension that results should use.
     *
     * For example, if the spreadsheet data is: `A1=1,B1=2,A2=3,B2=4`, then
     * requesting `range=A1:B2,majorDimension=ROWS` will return `[[1,2],[3,4]]`,
     * whereas requesting `range=A1:B2,majorDimension=COLUMNS` will return
     * `[[1,3],[2,4]]`.
     * @opt_param string valueRenderOption How values should be represented in the
     * output. The default render option is ValueRenderOption.FORMATTED_VALUE.
     * @opt_param string dateTimeRenderOption How dates, times, and durations should
     * be represented in the output. This is ignored if value_render_option is
     * FORMATTED_VALUE. The default dateTime render option is
     * [DateTimeRenderOption.SERIAL_NUMBER].
     * @return Google_Service_Sheets_ValueRange
     */
    public function get($spreadsheetId, $range, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'range' => $range);
        $params = array_merge($params, $optParams);
        return $this->call('get', array($params), "Google_Service_Sheets_ValueRange");
    }
    /**
     * Sets values in a range of a spreadsheet. The caller must specify the
     * spreadsheet ID, range, and a valueInputOption. (values.update)
     *
     * @param string $spreadsheetId The ID of the spreadsheet to update.
     * @param string $range The A1 notation of the values to update.
     * @param Google_Service_Sheets_ValueRange $postBody
     * @param array $optParams Optional parameters.
     *
     * @opt_param string responseValueRenderOption Determines how values in the
     * response should be rendered. The default render option is
     * ValueRenderOption.FORMATTED_VALUE.
     * @opt_param string valueInputOption How the input data should be interpreted.
     * @opt_param string responseDateTimeRenderOption Determines how dates, times,
     * and durations in the response should be rendered. This is ignored if
     * response_value_render_option is FORMATTED_VALUE. The default dateTime render
     * option is DateTimeRenderOption.SERIAL_NUMBER.
     * @opt_param bool includeValuesInResponse Determines if the update response
     * should include the values of the cells that were updated. By default,
     * responses do not include the updated values. If the range to write was larger
     * than than the range actually written, the response will include all values in
     * the requested range (excluding trailing empty rows and columns).
     * @return Google_Service_Sheets_UpdateValuesResponse
     */
    public function update($spreadsheetId, $range, Google_Service_Sheets_ValueRange $postBody, $optParams = array())
    {
        $params = array('spreadsheetId' => $spreadsheetId, 'range' => $range, 'postBody' => $postBody);
        $params = array_merge($params, $optParams);

        return $this->call('update', array($params), "Google_Service_Sheets_UpdateValuesResponse");
    }
}



class Google_Service_Sheets_BatchUpdateSpreadsheetRequest extends Google_Collection
{
    protected $collection_key = 'responseRanges';
    public $includeSpreadsheetInResponse;
    protected $requestsType = 'Google_Service_Sheets_Request';
    protected $requestsDataType = 'array';
    public $responseIncludeGridData;
    public $responseRanges;

    public function setIncludeSpreadsheetInResponse($includeSpreadsheetInResponse)
    {
        $this->includeSpreadsheetInResponse = $includeSpreadsheetInResponse;
    }
    public function getIncludeSpreadsheetInResponse()
    {
        return $this->includeSpreadsheetInResponse;
    }
    /**
     * @param Google_Service_Sheets_Request
     */
    public function setRequests($requests)
    {
        $this->requests = $requests;
    }
    /**
     * @return Google_Service_Sheets_Request
     */
    public function getRequests()
    {
        return $this->requests;
    }
    public function setResponseIncludeGridData($responseIncludeGridData)
    {
        $this->responseIncludeGridData = $responseIncludeGridData;
    }
    public function getResponseIncludeGridData()
    {
        return $this->responseIncludeGridData;
    }
    public function setResponseRanges($responseRanges)
    {
        $this->responseRanges = $responseRanges;
    }
    public function getResponseRanges()
    {
        return $this->responseRanges;
    }
}

class Google_Service_Sheets_BatchUpdateSpreadsheetResponse extends Google_Collection
{
    protected $collection_key = 'replies';
    protected $repliesType = 'Google_Service_Sheets_Response';
    protected $repliesDataType = 'array';
    public $spreadsheetId;
    protected $updatedSpreadsheetType = 'Google_Service_Sheets_Spreadsheet';
    protected $updatedSpreadsheetDataType = '';

    /**
     * @param Google_Service_Sheets_Response
     */
    public function setReplies($replies)
    {
        $this->replies = $replies;
    }
    /**
     * @return Google_Service_Sheets_Response
     */
    public function getReplies()
    {
        return $this->replies;
    }
    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
    /**
     * @param Google_Service_Sheets_Spreadsheet
     */
    public function setUpdatedSpreadsheet(Google_Service_Sheets_Spreadsheet $updatedSpreadsheet)
    {
        $this->updatedSpreadsheet = $updatedSpreadsheet;
    }
    /**
     * @return Google_Service_Sheets_Spreadsheet
     */
    public function getUpdatedSpreadsheet()
    {
        return $this->updatedSpreadsheet;
    }
}

class Google_Service_Sheets_Spreadsheet extends Google_Collection
{
    protected $collection_key = 'sheets';
    protected $developerMetadataType = 'Google_Service_Sheets_DeveloperMetadata';
    protected $developerMetadataDataType = 'array';
    protected $namedRangesType = 'Google_Service_Sheets_NamedRange';
    protected $namedRangesDataType = 'array';
    protected $propertiesType = 'Google_Service_Sheets_SpreadsheetProperties';
    protected $propertiesDataType = '';
    protected $sheetsType = 'Google_Service_Sheets_Sheet';
    protected $sheetsDataType = 'array';
    public $spreadsheetId;
    public $spreadsheetUrl;

    /**
     * @param Google_Service_Sheets_DeveloperMetadata
     */
    public function setDeveloperMetadata($developerMetadata)
    {
        $this->developerMetadata = $developerMetadata;
    }
    /**
     * @return Google_Service_Sheets_DeveloperMetadata
     */
    public function getDeveloperMetadata()
    {
        return $this->developerMetadata;
    }
    /**
     * @param Google_Service_Sheets_NamedRange
     */
    public function setNamedRanges($namedRanges)
    {
        $this->namedRanges = $namedRanges;
    }
    /**
     * @return Google_Service_Sheets_NamedRange
     */
    public function getNamedRanges()
    {
        return $this->namedRanges;
    }
    /**
     * @param Google_Service_Sheets_SpreadsheetProperties
     */
    public function setProperties(Google_Service_Sheets_SpreadsheetProperties $properties)
    {
        $this->properties = $properties;
    }
    /**
     * @return Google_Service_Sheets_SpreadsheetProperties
     */
    public function getProperties()
    {
        return $this->properties;
    }
    /**
     * @param Google_Service_Sheets_Sheet
     */
    public function setSheets($sheets)
    {
        $this->sheets = $sheets;
    }
    /**
     * @return Google_Service_Sheets_Sheet
     */
    public function getSheets()
    {
        return $this->sheets;
    }
    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
    public function setSpreadsheetUrl($spreadsheetUrl)
    {
        $this->spreadsheetUrl = $spreadsheetUrl;
    }
    public function getSpreadsheetUrl()
    {
        return $this->spreadsheetUrl;
    }
}

class Google_Service_Sheets_GetSpreadsheetByDataFilterRequest extends Google_Collection
{
    protected $collection_key = 'dataFilters';
    protected $dataFiltersType = 'Google_Service_Sheets_DataFilter';
    protected $dataFiltersDataType = 'array';
    public $includeGridData;

    /**
     * @param Google_Service_Sheets_DataFilter
     */
    public function setDataFilters($dataFilters)
    {
        $this->dataFilters = $dataFilters;
    }
    /**
     * @return Google_Service_Sheets_DataFilter
     */
    public function getDataFilters()
    {
        return $this->dataFilters;
    }
    public function setIncludeGridData($includeGridData)
    {
        $this->includeGridData = $includeGridData;
    }
    public function getIncludeGridData()
    {
        return $this->includeGridData;
    }
}

class Google_Service_Sheets_SearchDeveloperMetadataRequest extends Google_Collection
{
    protected $collection_key = 'dataFilters';
    protected $dataFiltersType = 'Google_Service_Sheets_DataFilter';
    protected $dataFiltersDataType = 'array';

    /**
     * @param Google_Service_Sheets_DataFilter
     */
    public function setDataFilters($dataFilters)
    {
        $this->dataFilters = $dataFilters;
    }
    /**
     * @return Google_Service_Sheets_DataFilter
     */
    public function getDataFilters()
    {
        return $this->dataFilters;
    }
}

class Google_Service_Sheets_SearchDeveloperMetadataResponse extends Google_Collection
{
    protected $collection_key = 'matchedDeveloperMetadata';
    protected $matchedDeveloperMetadataType = 'Google_Service_Sheets_MatchedDeveloperMetadata';
    protected $matchedDeveloperMetadataDataType = 'array';

    /**
     * @param Google_Service_Sheets_MatchedDeveloperMetadata
     */
    public function setMatchedDeveloperMetadata($matchedDeveloperMetadata)
    {
        $this->matchedDeveloperMetadata = $matchedDeveloperMetadata;
    }
    /**
     * @return Google_Service_Sheets_MatchedDeveloperMetadata
     */
    public function getMatchedDeveloperMetadata()
    {
        return $this->matchedDeveloperMetadata;
    }
}

class Google_Service_Sheets_DeveloperMetadata extends Google_Model
{
    protected $locationType = 'Google_Service_Sheets_DeveloperMetadataLocation';
    protected $locationDataType = '';
    public $metadataId;
    public $metadataKey;
    public $metadataValue;
    public $visibility;

    /**
     * @param Google_Service_Sheets_DeveloperMetadataLocation
     */
    public function setLocation(Google_Service_Sheets_DeveloperMetadataLocation $location)
    {
        $this->location = $location;
    }
    /**
     * @return Google_Service_Sheets_DeveloperMetadataLocation
     */
    public function getLocation()
    {
        return $this->location;
    }
    public function setMetadataId($metadataId)
    {
        $this->metadataId = $metadataId;
    }
    public function getMetadataId()
    {
        return $this->metadataId;
    }
    public function setMetadataKey($metadataKey)
    {
        $this->metadataKey = $metadataKey;
    }
    public function getMetadataKey()
    {
        return $this->metadataKey;
    }
    public function setMetadataValue($metadataValue)
    {
        $this->metadataValue = $metadataValue;
    }
    public function getMetadataValue()
    {
        return $this->metadataValue;
    }
    public function setVisibility($visibility)
    {
        $this->visibility = $visibility;
    }
    public function getVisibility()
    {
        return $this->visibility;
    }
}

class Google_Service_Sheets_ValueRange extends Google_Collection
{
    protected $collection_key = 'values';
    public $majorDimension;
    public $range;
    public $values;

    public function setMajorDimension($majorDimension)
    {
        $this->majorDimension = $majorDimension;
    }
    public function getMajorDimension()
    {
        return $this->majorDimension;
    }
    public function setRange($range)
    {
        $this->range = $range;
    }
    public function getRange()
    {
        return $this->range;
    }
    public function setValues($values)
    {
        $this->values = $values;
    }
    public function getValues()
    {
        return $this->values;
    }
}

class Google_Service_Sheets_AppendValuesResponse extends Google_Model
{
    public $spreadsheetId;
    public $tableRange;
    protected $updatesType = 'Google_Service_Sheets_UpdateValuesResponse';
    protected $updatesDataType = '';

    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
    public function setTableRange($tableRange)
    {
        $this->tableRange = $tableRange;
    }
    public function getTableRange()
    {
        return $this->tableRange;
    }
    /**
     * @param Google_Service_Sheets_UpdateValuesResponse
     */
    public function setUpdates(Google_Service_Sheets_UpdateValuesResponse $updates)
    {
        $this->updates = $updates;
    }
    /**
     * @return Google_Service_Sheets_UpdateValuesResponse
     */
    public function getUpdates()
    {
        return $this->updates;
    }
}

class Google_Service_Sheets_BatchClearValuesRequest extends Google_Collection
{
    protected $collection_key = 'ranges';
    public $ranges;

    public function setRanges($ranges)
    {
        $this->ranges = $ranges;
    }
    public function getRanges()
    {
        return $this->ranges;
    }
}

class Google_Service_Sheets_BatchClearValuesResponse extends Google_Collection
{
    protected $collection_key = 'clearedRanges';
    public $clearedRanges;
    public $spreadsheetId;

    public function setClearedRanges($clearedRanges)
    {
        $this->clearedRanges = $clearedRanges;
    }
    public function getClearedRanges()
    {
        return $this->clearedRanges;
    }
    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
}

class Google_Service_Sheets_BatchClearValuesByDataFilterRequest extends Google_Collection
{
    protected $collection_key = 'dataFilters';
    protected $dataFiltersType = 'Google_Service_Sheets_DataFilter';
    protected $dataFiltersDataType = 'array';

    /**
     * @param Google_Service_Sheets_DataFilter
     */
    public function setDataFilters($dataFilters)
    {
        $this->dataFilters = $dataFilters;
    }
    /**
     * @return Google_Service_Sheets_DataFilter
     */
    public function getDataFilters()
    {
        return $this->dataFilters;
    }
}

class Google_Service_Sheets_BatchClearValuesByDataFilterResponse extends Google_Collection
{
    protected $collection_key = 'clearedRanges';
    public $clearedRanges;
    public $spreadsheetId;

    public function setClearedRanges($clearedRanges)
    {
        $this->clearedRanges = $clearedRanges;
    }
    public function getClearedRanges()
    {
        return $this->clearedRanges;
    }
    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
}

class Google_Service_Sheets_BatchGetValuesResponse extends Google_Collection
{
    protected $collection_key = 'valueRanges';
    public $spreadsheetId;
    protected $valueRangesType = 'Google_Service_Sheets_ValueRange';
    protected $valueRangesDataType = 'array';

    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
    /**
     * @param Google_Service_Sheets_ValueRange
     */
    public function setValueRanges($valueRanges)
    {
        $this->valueRanges = $valueRanges;
    }
    /**
     * @return Google_Service_Sheets_ValueRange
     */
    public function getValueRanges()
    {
        return $this->valueRanges;
    }
}

class Google_Service_Sheets_BatchGetValuesByDataFilterRequest extends Google_Collection
{
    protected $collection_key = 'dataFilters';
    protected $dataFiltersType = 'Google_Service_Sheets_DataFilter';
    protected $dataFiltersDataType = 'array';
    public $dateTimeRenderOption;
    public $majorDimension;
    public $valueRenderOption;

    /**
     * @param Google_Service_Sheets_DataFilter
     */
    public function setDataFilters($dataFilters)
    {
        $this->dataFilters = $dataFilters;
    }
    /**
     * @return Google_Service_Sheets_DataFilter
     */
    public function getDataFilters()
    {
        return $this->dataFilters;
    }
    public function setDateTimeRenderOption($dateTimeRenderOption)
    {
        $this->dateTimeRenderOption = $dateTimeRenderOption;
    }
    public function getDateTimeRenderOption()
    {
        return $this->dateTimeRenderOption;
    }
    public function setMajorDimension($majorDimension)
    {
        $this->majorDimension = $majorDimension;
    }
    public function getMajorDimension()
    {
        return $this->majorDimension;
    }
    public function setValueRenderOption($valueRenderOption)
    {
        $this->valueRenderOption = $valueRenderOption;
    }
    public function getValueRenderOption()
    {
        return $this->valueRenderOption;
    }
}

class Google_Service_Sheets_BatchGetValuesByDataFilterResponse extends Google_Collection
{
    protected $collection_key = 'valueRanges';
    public $spreadsheetId;
    protected $valueRangesType = 'Google_Service_Sheets_MatchedValueRange';
    protected $valueRangesDataType = 'array';

    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
    /**
     * @param Google_Service_Sheets_MatchedValueRange
     */
    public function setValueRanges($valueRanges)
    {
        $this->valueRanges = $valueRanges;
    }
    /**
     * @return Google_Service_Sheets_MatchedValueRange
     */
    public function getValueRanges()
    {
        return $this->valueRanges;
    }
}

class Google_Service_Sheets_BatchUpdateValuesRequest extends Google_Collection
{
    protected $collection_key = 'data';
    protected $dataType = 'Google_Service_Sheets_ValueRange';
    protected $dataDataType = 'array';
    public $includeValuesInResponse;
    public $responseDateTimeRenderOption;
    public $responseValueRenderOption;
    public $valueInputOption;

    /**
     * @param Google_Service_Sheets_ValueRange
     */
    public function setData($data)
    {
        $this->data = $data;
    }
    /**
     * @return Google_Service_Sheets_ValueRange
     */
    public function getData()
    {
        return $this->data;
    }
    public function setIncludeValuesInResponse($includeValuesInResponse)
    {
        $this->includeValuesInResponse = $includeValuesInResponse;
    }
    public function getIncludeValuesInResponse()
    {
        return $this->includeValuesInResponse;
    }
    public function setResponseDateTimeRenderOption($responseDateTimeRenderOption)
    {
        $this->responseDateTimeRenderOption = $responseDateTimeRenderOption;
    }
    public function getResponseDateTimeRenderOption()
    {
        return $this->responseDateTimeRenderOption;
    }
    public function setResponseValueRenderOption($responseValueRenderOption)
    {
        $this->responseValueRenderOption = $responseValueRenderOption;
    }
    public function getResponseValueRenderOption()
    {
        return $this->responseValueRenderOption;
    }
    public function setValueInputOption($valueInputOption)
    {
        $this->valueInputOption = $valueInputOption;
    }
    public function getValueInputOption()
    {
        return $this->valueInputOption;
    }
}

class Google_Service_Sheets_BatchUpdateValuesResponse extends Google_Collection
{
    protected $collection_key = 'responses';
    protected $responsesType = 'Google_Service_Sheets_UpdateValuesResponse';
    protected $responsesDataType = 'array';
    public $spreadsheetId;
    public $totalUpdatedCells;
    public $totalUpdatedColumns;
    public $totalUpdatedRows;
    public $totalUpdatedSheets;

    /**
     * @param Google_Service_Sheets_UpdateValuesResponse
     */
    public function setResponses($responses)
    {
        $this->responses = $responses;
    }
    /**
     * @return Google_Service_Sheets_UpdateValuesResponse
     */
    public function getResponses()
    {
        return $this->responses;
    }
    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
    public function setTotalUpdatedCells($totalUpdatedCells)
    {
        $this->totalUpdatedCells = $totalUpdatedCells;
    }
    public function getTotalUpdatedCells()
    {
        return $this->totalUpdatedCells;
    }
    public function setTotalUpdatedColumns($totalUpdatedColumns)
    {
        $this->totalUpdatedColumns = $totalUpdatedColumns;
    }
    public function getTotalUpdatedColumns()
    {
        return $this->totalUpdatedColumns;
    }
    public function setTotalUpdatedRows($totalUpdatedRows)
    {
        $this->totalUpdatedRows = $totalUpdatedRows;
    }
    public function getTotalUpdatedRows()
    {
        return $this->totalUpdatedRows;
    }
    public function setTotalUpdatedSheets($totalUpdatedSheets)
    {
        $this->totalUpdatedSheets = $totalUpdatedSheets;
    }
    public function getTotalUpdatedSheets()
    {
        return $this->totalUpdatedSheets;
    }
}

class Google_Service_Sheets_BatchUpdateValuesByDataFilterRequest extends Google_Collection
{
    protected $collection_key = 'data';
    protected $dataType = 'Google_Service_Sheets_DataFilterValueRange';
    protected $dataDataType = 'array';
    public $includeValuesInResponse;
    public $responseDateTimeRenderOption;
    public $responseValueRenderOption;
    public $valueInputOption;

    /**
     * @param Google_Service_Sheets_DataFilterValueRange
     */
    public function setData($data)
    {
        $this->data = $data;
    }
    /**
     * @return Google_Service_Sheets_DataFilterValueRange
     */
    public function getData()
    {
        return $this->data;
    }
    public function setIncludeValuesInResponse($includeValuesInResponse)
    {
        $this->includeValuesInResponse = $includeValuesInResponse;
    }
    public function getIncludeValuesInResponse()
    {
        return $this->includeValuesInResponse;
    }
    public function setResponseDateTimeRenderOption($responseDateTimeRenderOption)
    {
        $this->responseDateTimeRenderOption = $responseDateTimeRenderOption;
    }
    public function getResponseDateTimeRenderOption()
    {
        return $this->responseDateTimeRenderOption;
    }
    public function setResponseValueRenderOption($responseValueRenderOption)
    {
        $this->responseValueRenderOption = $responseValueRenderOption;
    }
    public function getResponseValueRenderOption()
    {
        return $this->responseValueRenderOption;
    }
    public function setValueInputOption($valueInputOption)
    {
        $this->valueInputOption = $valueInputOption;
    }
    public function getValueInputOption()
    {
        return $this->valueInputOption;
    }
}

class Google_Service_Sheets_BatchUpdateValuesByDataFilterResponse extends Google_Collection
{
    protected $collection_key = 'responses';
    protected $responsesType = 'Google_Service_Sheets_UpdateValuesByDataFilterResponse';
    protected $responsesDataType = 'array';
    public $spreadsheetId;
    public $totalUpdatedCells;
    public $totalUpdatedColumns;
    public $totalUpdatedRows;
    public $totalUpdatedSheets;

    /**
     * @param Google_Service_Sheets_UpdateValuesByDataFilterResponse
     */
    public function setResponses($responses)
    {
        $this->responses = $responses;
    }
    /**
     * @return Google_Service_Sheets_UpdateValuesByDataFilterResponse
     */
    public function getResponses()
    {
        return $this->responses;
    }
    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
    public function setTotalUpdatedCells($totalUpdatedCells)
    {
        $this->totalUpdatedCells = $totalUpdatedCells;
    }
    public function getTotalUpdatedCells()
    {
        return $this->totalUpdatedCells;
    }
    public function setTotalUpdatedColumns($totalUpdatedColumns)
    {
        $this->totalUpdatedColumns = $totalUpdatedColumns;
    }
    public function getTotalUpdatedColumns()
    {
        return $this->totalUpdatedColumns;
    }
    public function setTotalUpdatedRows($totalUpdatedRows)
    {
        $this->totalUpdatedRows = $totalUpdatedRows;
    }
    public function getTotalUpdatedRows()
    {
        return $this->totalUpdatedRows;
    }
    public function setTotalUpdatedSheets($totalUpdatedSheets)
    {
        $this->totalUpdatedSheets = $totalUpdatedSheets;
    }
    public function getTotalUpdatedSheets()
    {
        return $this->totalUpdatedSheets;
    }
}

class Google_Service_Sheets_ClearValuesRequest extends Google_Model
{
}

class Google_Service_Sheets_ClearValuesResponse extends Google_Model
{
    public $clearedRange;
    public $spreadsheetId;

    public function setClearedRange($clearedRange)
    {
        $this->clearedRange = $clearedRange;
    }
    public function getClearedRange()
    {
        return $this->clearedRange;
    }
    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
}

class Google_Service_Sheets_UpdateValuesResponse extends Google_Model
{
    public $spreadsheetId;
    public $updatedCells;
    public $updatedColumns;
    protected $updatedDataType = 'Google_Service_Sheets_ValueRange';
    protected $updatedDataDataType = '';
    public $updatedRange;
    public $updatedRows;

    public function setSpreadsheetId($spreadsheetId)
    {
        $this->spreadsheetId = $spreadsheetId;
    }
    public function getSpreadsheetId()
    {
        return $this->spreadsheetId;
    }
    public function setUpdatedCells($updatedCells)
    {
        $this->updatedCells = $updatedCells;
    }
    public function getUpdatedCells()
    {
        return $this->updatedCells;
    }
    public function setUpdatedColumns($updatedColumns)
    {
        $this->updatedColumns = $updatedColumns;
    }
    public function getUpdatedColumns()
    {
        return $this->updatedColumns;
    }
    /**
     * @param Google_Service_Sheets_ValueRange
     */
    public function setUpdatedData(Google_Service_Sheets_ValueRange $updatedData)
    {
        $this->updatedData = $updatedData;
    }
    /**
     * @return Google_Service_Sheets_ValueRange
     */
    public function getUpdatedData()
    {
        return $this->updatedData;
    }
    public function setUpdatedRange($updatedRange)
    {
        $this->updatedRange = $updatedRange;
    }
    public function getUpdatedRange()
    {
        return $this->updatedRange;
    }
    public function setUpdatedRows($updatedRows)
    {
        $this->updatedRows = $updatedRows;
    }
    public function getUpdatedRows()
    {
        return $this->updatedRows;
    }
}

class Google_Service_Sheets_SpreadsheetProperties extends Google_Model
{
    public $autoRecalc;
    protected $defaultFormatType = 'Google_Service_Sheets_CellFormat';
    protected $defaultFormatDataType = '';
    protected $iterativeCalculationSettingsType = 'Google_Service_Sheets_IterativeCalculationSettings';
    protected $iterativeCalculationSettingsDataType = '';
    public $locale;
    public $timeZone;
    public $title;

    public function setAutoRecalc($autoRecalc)
    {
        $this->autoRecalc = $autoRecalc;
    }
    public function getAutoRecalc()
    {
        return $this->autoRecalc;
    }
    /**
     * @param Google_Service_Sheets_CellFormat
     */
    public function setDefaultFormat(Google_Service_Sheets_CellFormat $defaultFormat)
    {
        $this->defaultFormat = $defaultFormat;
    }
    /**
     * @return Google_Service_Sheets_CellFormat
     */
    public function getDefaultFormat()
    {
        return $this->defaultFormat;
    }
    /**
     * @param Google_Service_Sheets_IterativeCalculationSettings
     */
    public function setIterativeCalculationSettings(Google_Service_Sheets_IterativeCalculationSettings $iterativeCalculationSettings)
    {
        $this->iterativeCalculationSettings = $iterativeCalculationSettings;
    }
    /**
     * @return Google_Service_Sheets_IterativeCalculationSettings
     */
    public function getIterativeCalculationSettings()
    {
        return $this->iterativeCalculationSettings;
    }
    public function setLocale($locale)
    {
        $this->locale = $locale;
    }
    public function getLocale()
    {
        return $this->locale;
    }
    public function setTimeZone($timeZone)
    {
        $this->timeZone = $timeZone;
    }
    public function getTimeZone()
    {
        return $this->timeZone;
    }
    public function setTitle($title)
    {
        $this->title = $title;
    }
    public function getTitle()
    {
        return $this->title;
    }
}

class Google_Service_Sheets_DeveloperMetadataLocation extends Google_Model
{
    protected $dimensionRangeType = 'Google_Service_Sheets_DimensionRange';
    protected $dimensionRangeDataType = '';
    public $locationType;
    public $sheetId;
    public $spreadsheet;

    /**
     * @param Google_Service_Sheets_DimensionRange
     */
    public function setDimensionRange(Google_Service_Sheets_DimensionRange $dimensionRange)
    {
        $this->dimensionRange = $dimensionRange;
    }
    /**
     * @return Google_Service_Sheets_DimensionRange
     */
    public function getDimensionRange()
    {
        return $this->dimensionRange;
    }
    public function setLocationType($locationType)
    {
        $this->locationType = $locationType;
    }
    public function getLocationType()
    {
        return $this->locationType;
    }
    public function setSheetId($sheetId)
    {
        $this->sheetId = $sheetId;
    }
    public function getSheetId()
    {
        return $this->sheetId;
    }
    public function setSpreadsheet($spreadsheet)
    {
        $this->spreadsheet = $spreadsheet;
    }
    public function getSpreadsheet()
    {
        return $this->spreadsheet;
    }
}

class Google_Service_Sheets_DimensionRange extends Google_Model
{
    public $dimension;
    public $endIndex;
    public $sheetId;
    public $startIndex;

    public function setDimension($dimension)
    {
        $this->dimension = $dimension;
    }
    public function getDimension()
    {
        return $this->dimension;
    }
    public function setEndIndex($endIndex)
    {
        $this->endIndex = $endIndex;
    }
    public function getEndIndex()
    {
        return $this->endIndex;
    }
    public function setSheetId($sheetId)
    {
        $this->sheetId = $sheetId;
    }
    public function getSheetId()
    {
        return $this->sheetId;
    }
    public function setStartIndex($startIndex)
    {
        $this->startIndex = $startIndex;
    }
    public function getStartIndex()
    {
        return $this->startIndex;
    }
}

class Google_Service_Sheets_CellFormat extends Google_Model
{
    protected $backgroundColorType = 'Google_Service_Sheets_Color';
    protected $backgroundColorDataType = '';
    protected $bordersType = 'Google_Service_Sheets_Borders';
    protected $bordersDataType = '';
    public $horizontalAlignment;
    public $hyperlinkDisplayType;
    protected $numberFormatType = 'Google_Service_Sheets_NumberFormat';
    protected $numberFormatDataType = '';
    protected $paddingType = 'Google_Service_Sheets_Padding';
    protected $paddingDataType = '';
    public $textDirection;
    protected $textFormatType = 'Google_Service_Sheets_TextFormat';
    protected $textFormatDataType = '';
    protected $textRotationType = 'Google_Service_Sheets_TextRotation';
    protected $textRotationDataType = '';
    public $verticalAlignment;
    public $wrapStrategy;

    /**
     * @param Google_Service_Sheets_Color
     */
    public function setBackgroundColor(Google_Service_Sheets_Color $backgroundColor)
    {
        $this->backgroundColor = $backgroundColor;
    }
    /**
     * @return Google_Service_Sheets_Color
     */
    public function getBackgroundColor()
    {
        return $this->backgroundColor;
    }
    /**
     * @param Google_Service_Sheets_Borders
     */
    public function setBorders(Google_Service_Sheets_Borders $borders)
    {
        $this->borders = $borders;
    }
    /**
     * @return Google_Service_Sheets_Borders
     */
    public function getBorders()
    {
        return $this->borders;
    }
    public function setHorizontalAlignment($horizontalAlignment)
    {
        $this->horizontalAlignment = $horizontalAlignment;
    }
    public function getHorizontalAlignment()
    {
        return $this->horizontalAlignment;
    }
    public function setHyperlinkDisplayType($hyperlinkDisplayType)
    {
        $this->hyperlinkDisplayType = $hyperlinkDisplayType;
    }
    public function getHyperlinkDisplayType()
    {
        return $this->hyperlinkDisplayType;
    }
    /**
     * @param Google_Service_Sheets_NumberFormat
     */
    public function setNumberFormat(Google_Service_Sheets_NumberFormat $numberFormat)
    {
        $this->numberFormat = $numberFormat;
    }
    /**
     * @return Google_Service_Sheets_NumberFormat
     */
    public function getNumberFormat()
    {
        return $this->numberFormat;
    }
    /**
     * @param Google_Service_Sheets_Padding
     */
    public function setPadding(Google_Service_Sheets_Padding $padding)
    {
        $this->padding = $padding;
    }
    /**
     * @return Google_Service_Sheets_Padding
     */
    public function getPadding()
    {
        return $this->padding;
    }
    public function setTextDirection($textDirection)
    {
        $this->textDirection = $textDirection;
    }
    public function getTextDirection()
    {
        return $this->textDirection;
    }
    /**
     * @param Google_Service_Sheets_TextFormat
     */
    public function setTextFormat(Google_Service_Sheets_TextFormat $textFormat)
    {
        $this->textFormat = $textFormat;
    }
    /**
     * @return Google_Service_Sheets_TextFormat
     */
    public function getTextFormat()
    {
        return $this->textFormat;
    }
    /**
     * @param Google_Service_Sheets_TextRotation
     */
    public function setTextRotation(Google_Service_Sheets_TextRotation $textRotation)
    {
        $this->textRotation = $textRotation;
    }
    /**
     * @return Google_Service_Sheets_TextRotation
     */
    public function getTextRotation()
    {
        return $this->textRotation;
    }
    public function setVerticalAlignment($verticalAlignment)
    {
        $this->verticalAlignment = $verticalAlignment;
    }
    public function getVerticalAlignment()
    {
        return $this->verticalAlignment;
    }
    public function setWrapStrategy($wrapStrategy)
    {
        $this->wrapStrategy = $wrapStrategy;
    }
    public function getWrapStrategy()
    {
        return $this->wrapStrategy;
    }
}

class Google_Service_Sheets_IterativeCalculationSettings extends Google_Model
{
    public $convergenceThreshold;
    public $maxIterations;

    public function setConvergenceThreshold($convergenceThreshold)
    {
        $this->convergenceThreshold = $convergenceThreshold;
    }
    public function getConvergenceThreshold()
    {
        return $this->convergenceThreshold;
    }
    public function setMaxIterations($maxIterations)
    {
        $this->maxIterations = $maxIterations;
    }
    public function getMaxIterations()
    {
        return $this->maxIterations;
    }
}

class Google_Service_Sheets_Color extends Google_Model
{
    public $alpha;
    public $blue;
    public $green;
    public $red;

    public function setAlpha($alpha)
    {
        $this->alpha = $alpha;
    }
    public function getAlpha()
    {
        return $this->alpha;
    }
    public function setBlue($blue)
    {
        $this->blue = $blue;
    }
    public function getBlue()
    {
        return $this->blue;
    }
    public function setGreen($green)
    {
        $this->green = $green;
    }
    public function getGreen()
    {
        return $this->green;
    }
    public function setRed($red)
    {
        $this->red = $red;
    }
    public function getRed()
    {
        return $this->red;
    }
}

class Google_Service_Sheets_Borders extends Google_Model
{
    protected $bottomType = 'Google_Service_Sheets_Border';
    protected $bottomDataType = '';
    protected $leftType = 'Google_Service_Sheets_Border';
    protected $leftDataType = '';
    protected $rightType = 'Google_Service_Sheets_Border';
    protected $rightDataType = '';
    protected $topType = 'Google_Service_Sheets_Border';
    protected $topDataType = '';

    /**
     * @param Google_Service_Sheets_Border
     */
    public function setBottom(Google_Service_Sheets_Border $bottom)
    {
        $this->bottom = $bottom;
    }
    /**
     * @return Google_Service_Sheets_Border
     */
    public function getBottom()
    {
        return $this->bottom;
    }
    /**
     * @param Google_Service_Sheets_Border
     */
    public function setLeft(Google_Service_Sheets_Border $left)
    {
        $this->left = $left;
    }
    /**
     * @return Google_Service_Sheets_Border
     */
    public function getLeft()
    {
        return $this->left;
    }
    /**
     * @param Google_Service_Sheets_Border
     */
    public function setRight(Google_Service_Sheets_Border $right)
    {
        $this->right = $right;
    }
    /**
     * @return Google_Service_Sheets_Border
     */
    public function getRight()
    {
        return $this->right;
    }
    /**
     * @param Google_Service_Sheets_Border
     */
    public function setTop(Google_Service_Sheets_Border $top)
    {
        $this->top = $top;
    }
    /**
     * @return Google_Service_Sheets_Border
     */
    public function getTop()
    {
        return $this->top;
    }
}

class Google_Service_Sheets_Border extends Google_Model
{
    protected $colorType = 'Google_Service_Sheets_Color';
    protected $colorDataType = '';
    public $style;
    public $width;

    /**
     * @param Google_Service_Sheets_Color
     */
    public function setColor(Google_Service_Sheets_Color $color)
    {
        $this->color = $color;
    }
    /**
     * @return Google_Service_Sheets_Color
     */
    public function getColor()
    {
        return $this->color;
    }
    public function setStyle($style)
    {
        $this->style = $style;
    }
    public function getStyle()
    {
        return $this->style;
    }
    public function setWidth($width)
    {
        $this->width = $width;
    }
    public function getWidth()
    {
        return $this->width;
    }
}

class Google_Service_Sheets_NumberFormat extends Google_Model
{
    public $pattern;
    public $type;

    public function setPattern($pattern)
    {
        $this->pattern = $pattern;
    }
    public function getPattern()
    {
        return $this->pattern;
    }
    public function setType($type)
    {
        $this->type = $type;
    }
    public function getType()
    {
        return $this->type;
    }
}

class Google_Service_Sheets_Padding extends Google_Model
{
    public $bottom;
    public $left;
    public $right;
    public $top;

    public function setBottom($bottom)
    {
        $this->bottom = $bottom;
    }
    public function getBottom()
    {
        return $this->bottom;
    }
    public function setLeft($left)
    {
        $this->left = $left;
    }
    public function getLeft()
    {
        return $this->left;
    }
    public function setRight($right)
    {
        $this->right = $right;
    }
    public function getRight()
    {
        return $this->right;
    }
    public function setTop($top)
    {
        $this->top = $top;
    }
    public function getTop()
    {
        return $this->top;
    }
}

class Google_Service_Sheets_TextFormat extends Google_Model
{
    public $bold;
    public $fontFamily;
    public $fontSize;
    protected $foregroundColorType = 'Google_Service_Sheets_Color';
    protected $foregroundColorDataType = '';
    public $italic;
    public $strikethrough;
    public $underline;

    public function setBold($bold)
    {
        $this->bold = $bold;
    }
    public function getBold()
    {
        return $this->bold;
    }
    public function setFontFamily($fontFamily)
    {
        $this->fontFamily = $fontFamily;
    }
    public function getFontFamily()
    {
        return $this->fontFamily;
    }
    public function setFontSize($fontSize)
    {
        $this->fontSize = $fontSize;
    }
    public function getFontSize()
    {
        return $this->fontSize;
    }
    /**
     * @param Google_Service_Sheets_Color
     */
    public function setForegroundColor(Google_Service_Sheets_Color $foregroundColor)
    {
        $this->foregroundColor = $foregroundColor;
    }
    /**
     * @return Google_Service_Sheets_Color
     */
    public function getForegroundColor()
    {
        return $this->foregroundColor;
    }
    public function setItalic($italic)
    {
        $this->italic = $italic;
    }
    public function getItalic()
    {
        return $this->italic;
    }
    public function setStrikethrough($strikethrough)
    {
        $this->strikethrough = $strikethrough;
    }
    public function getStrikethrough()
    {
        return $this->strikethrough;
    }
    public function setUnderline($underline)
    {
        $this->underline = $underline;
    }
    public function getUnderline()
    {
        return $this->underline;
    }
}

class Google_Service_Sheets_TextRotation extends Google_Model
{
    public $angle;
    public $vertical;

    public function setAngle($angle)
    {
        $this->angle = $angle;
    }
    public function getAngle()
    {
        return $this->angle;
    }
    public function setVertical($vertical)
    {
        $this->vertical = $vertical;
    }
    public function getVertical()
    {
        return $this->vertical;
    }
}


class Google_Service_Sheets_Sheet extends Google_Collection
{
    protected $collection_key = 'rowGroups';
    protected $bandedRangesType = 'Google_Service_Sheets_BandedRange';
    protected $bandedRangesDataType = 'array';
    protected $basicFilterType = 'Google_Service_Sheets_BasicFilter';
    protected $basicFilterDataType = '';
    protected $chartsType = 'Google_Service_Sheets_EmbeddedChart';
    protected $chartsDataType = 'array';
    protected $columnGroupsType = 'Google_Service_Sheets_DimensionGroup';
    protected $columnGroupsDataType = 'array';
    protected $conditionalFormatsType = 'Google_Service_Sheets_ConditionalFormatRule';
    protected $conditionalFormatsDataType = 'array';
    protected $dataType = 'Google_Service_Sheets_GridData';
    protected $dataDataType = 'array';
    protected $developerMetadataType = 'Google_Service_Sheets_DeveloperMetadata';
    protected $developerMetadataDataType = 'array';
    protected $filterViewsType = 'Google_Service_Sheets_FilterView';
    protected $filterViewsDataType = 'array';
    protected $mergesType = 'Google_Service_Sheets_GridRange';
    protected $mergesDataType = 'array';
    protected $propertiesType = 'Google_Service_Sheets_SheetProperties';
    protected $propertiesDataType = '';
    protected $protectedRangesType = 'Google_Service_Sheets_ProtectedRange';
    protected $protectedRangesDataType = 'array';
    protected $rowGroupsType = 'Google_Service_Sheets_DimensionGroup';
    protected $rowGroupsDataType = 'array';

    /**
     * @param Google_Service_Sheets_BandedRange
     */
    public function setBandedRanges($bandedRanges)
    {
        $this->bandedRanges = $bandedRanges;
    }
    /**
     * @return Google_Service_Sheets_BandedRange
     */
    public function getBandedRanges()
    {
        return $this->bandedRanges;
    }
    /**
     * @param Google_Service_Sheets_BasicFilter
     */
    public function setBasicFilter(Google_Service_Sheets_BasicFilter $basicFilter)
    {
        $this->basicFilter = $basicFilter;
    }
    /**
     * @return Google_Service_Sheets_BasicFilter
     */
    public function getBasicFilter()
    {
        return $this->basicFilter;
    }
    /**
     * @param Google_Service_Sheets_EmbeddedChart
     */
    public function setCharts($charts)
    {
        $this->charts = $charts;
    }
    /**
     * @return Google_Service_Sheets_EmbeddedChart
     */
    public function getCharts()
    {
        return $this->charts;
    }
    /**
     * @param Google_Service_Sheets_DimensionGroup
     */
    public function setColumnGroups($columnGroups)
    {
        $this->columnGroups = $columnGroups;
    }
    /**
     * @return Google_Service_Sheets_DimensionGroup
     */
    public function getColumnGroups()
    {
        return $this->columnGroups;
    }
    /**
     * @param Google_Service_Sheets_ConditionalFormatRule
     */
    public function setConditionalFormats($conditionalFormats)
    {
        $this->conditionalFormats = $conditionalFormats;
    }
    /**
     * @return Google_Service_Sheets_ConditionalFormatRule
     */
    public function getConditionalFormats()
    {
        return $this->conditionalFormats;
    }
    /**
     * @param Google_Service_Sheets_GridData
     */
    public function setData($data)
    {
        $this->data = $data;
    }
    /**
     * @return Google_Service_Sheets_GridData
     */
    public function getData()
    {
        return $this->data;
    }
    /**
     * @param Google_Service_Sheets_DeveloperMetadata
     */
    public function setDeveloperMetadata($developerMetadata)
    {
        $this->developerMetadata = $developerMetadata;
    }
    /**
     * @return Google_Service_Sheets_DeveloperMetadata
     */
    public function getDeveloperMetadata()
    {
        return $this->developerMetadata;
    }
    /**
     * @param Google_Service_Sheets_FilterView
     */
    public function setFilterViews($filterViews)
    {
        $this->filterViews = $filterViews;
    }
    /**
     * @return Google_Service_Sheets_FilterView
     */
    public function getFilterViews()
    {
        return $this->filterViews;
    }
    /**
     * @param Google_Service_Sheets_GridRange
     */
    public function setMerges($merges)
    {
        $this->merges = $merges;
    }
    /**
     * @return Google_Service_Sheets_GridRange
     */
    public function getMerges()
    {
        return $this->merges;
    }
    /**
     * @param Google_Service_Sheets_SheetProperties
     */
    public function setProperties(Google_Service_Sheets_SheetProperties $properties)
    {
        $this->properties = $properties;
    }
    /**
     * @return Google_Service_Sheets_SheetProperties
     */
    public function getProperties()
    {
        return $this->properties;
    }
    /**
     * @param Google_Service_Sheets_ProtectedRange
     */
    public function setProtectedRanges($protectedRanges)
    {
        $this->protectedRanges = $protectedRanges;
    }
    /**
     * @return Google_Service_Sheets_ProtectedRange
     */
    public function getProtectedRanges()
    {
        return $this->protectedRanges;
    }
    /**
     * @param Google_Service_Sheets_DimensionGroup
     */
    public function setRowGroups($rowGroups)
    {
        $this->rowGroups = $rowGroups;
    }
    /**
     * @return Google_Service_Sheets_DimensionGroup
     */
    public function getRowGroups()
    {
        return $this->rowGroups;
    }
}

class Google_Service_Sheets_SheetProperties extends Google_Model
{
    protected $gridPropertiesType = 'Google_Service_Sheets_GridProperties';
    protected $gridPropertiesDataType = '';
    public $hidden;
    public $index;
    public $rightToLeft;
    public $sheetId;
    public $sheetType;
    protected $tabColorType = 'Google_Service_Sheets_Color';
    protected $tabColorDataType = '';
    public $title;

    /**
     * @param Google_Service_Sheets_GridProperties
     */
    public function setGridProperties(Google_Service_Sheets_GridProperties $gridProperties)
    {
        $this->gridProperties = $gridProperties;
    }
    /**
     * @return Google_Service_Sheets_GridProperties
     */
    public function getGridProperties()
    {
        return $this->gridProperties;
    }
    public function setHidden($hidden)
    {
        $this->hidden = $hidden;
    }
    public function getHidden()
    {
        return $this->hidden;
    }
    public function setIndex($index)
    {
        $this->index = $index;
    }
    public function getIndex()
    {
        return $this->index;
    }
    public function setRightToLeft($rightToLeft)
    {
        $this->rightToLeft = $rightToLeft;
    }
    public function getRightToLeft()
    {
        return $this->rightToLeft;
    }
    public function setSheetId($sheetId)
    {
        $this->sheetId = $sheetId;
    }
    public function getSheetId()
    {
        return $this->sheetId;
    }
    public function setSheetType($sheetType)
    {
        $this->sheetType = $sheetType;
    }
    public function getSheetType()
    {
        return $this->sheetType;
    }
    /**
     * @param Google_Service_Sheets_Color
     */
    public function setTabColor(Google_Service_Sheets_Color $tabColor)
    {
        $this->tabColor = $tabColor;
    }
    /**
     * @return Google_Service_Sheets_Color
     */
    public function getTabColor()
    {
        return $this->tabColor;
    }
    public function setTitle($title)
    {
        $this->title = $title;
    }
    public function getTitle()
    {
        return $this->title;
    }
}

class Google_Service_Sheets_GridProperties extends Google_Model
{
    public $columnCount;
    public $columnGroupControlAfter;
    public $frozenColumnCount;
    public $frozenRowCount;
    public $hideGridlines;
    public $rowCount;
    public $rowGroupControlAfter;

    public function setColumnCount($columnCount)
    {
        $this->columnCount = $columnCount;
    }
    public function getColumnCount()
    {
        return $this->columnCount;
    }
    public function setColumnGroupControlAfter($columnGroupControlAfter)
    {
        $this->columnGroupControlAfter = $columnGroupControlAfter;
    }
    public function getColumnGroupControlAfter()
    {
        return $this->columnGroupControlAfter;
    }
    public function setFrozenColumnCount($frozenColumnCount)
    {
        $this->frozenColumnCount = $frozenColumnCount;
    }
    public function getFrozenColumnCount()
    {
        return $this->frozenColumnCount;
    }
    public function setFrozenRowCount($frozenRowCount)
    {
        $this->frozenRowCount = $frozenRowCount;
    }
    public function getFrozenRowCount()
    {
        return $this->frozenRowCount;
    }
    public function setHideGridlines($hideGridlines)
    {
        $this->hideGridlines = $hideGridlines;
    }
    public function getHideGridlines()
    {
        return $this->hideGridlines;
    }
    public function setRowCount($rowCount)
    {
        $this->rowCount = $rowCount;
    }
    public function getRowCount()
    {
        return $this->rowCount;
    }
    public function setRowGroupControlAfter($rowGroupControlAfter)
    {
        $this->rowGroupControlAfter = $rowGroupControlAfter;
    }
    public function getRowGroupControlAfter()
    {
        return $this->rowGroupControlAfter;
    }
}