<?php
/**
* @package RSform!Pro
* @copyright (C) www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

defined('_JEXEC') or die('Restricted access');

class RSFormGoogleSheets
{
	/*
	*	Google client ID
	*/
	protected $_clientID;
	
	/*
	*	Google secret
	*/
	protected $_secret;
	
	/* 
	*	stores the access token
	*/
	protected $access_token;
	
	/*
	*	Constructor
	*/
	
	protected $client;
	
	public function __construct() {
		require_once __DIR__ . '/Google/autoload.php';

		$this->getCredentials();
		
		$this->client = new Google_Client();
		$this->access_token = $this->getAccessToken();
		if ($this->access_token) {
			$this->client->setAccessToken($this->access_token);
		}
		if ($this->isInvalidToken()) {
			$this->refreshToken();
			$this->access_token = $this->getAccessToken();
		}

		if (!class_exists('RSFormProHelper') && file_exists(JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/rsform.php'))
		{
			require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/rsform.php';
		}
	}
	
	public static function getInstance() {
		static $instance = null;

		if ($instance === null)
		{
			$instance = new RSFormGoogleSheets();
		}

		return $instance;
	}
	
	protected function getCredentials() {
		$this->_clientID  = RSFormProHelper::getConfig('google.sheets.clientid');
		$this->_secret = RSFormProHelper::getConfig('google.sheets.secret');
	}
	
	/*
	*	Get auth URL
	*/
	public function getAuthURL($redirectUri) {
		$this->client->setClientId($this->_clientID);
		$this->client->setClientSecret($this->_secret);
		$this->client->setRedirectUri($redirectUri);
		$this->client->addScope(array(Google_Service_Sheets::SPREADSHEETS));
		$this->client->setAccessType('offline');
		$this->client->setApprovalPrompt('force');
		
		return $this->client->createAuthUrl();
	}
	
	public function getAccessToken()
	{
		if ($accesstoken = RSFormProHelper::getConfig('google.sheets.accesstoken'))
		{
			return $accesstoken;
		}

		return false;
	}
	
	public function revokeToken() {
		$status = false;
		if ($this->access_token) {
			try {
				$this->client->setClientId($this->_clientID);
				$this->client->setClientSecret($this->_secret);
				$revoked = $this->client->revokeToken();

				if ($revoked) {
					$this->storeAccessToken('');
					$this->access_token = false;
					$status = true;
				}
			} catch (Exception $e) {
				if (JFactory::getApplication()->isClient('administrator')) {
					JFactory::getApplication()->enqueueMessage(JText::sprintf('RSFP_GOOGLE_SHEETS_ERROR_NO_SHEETS', $e->getMessage()), 'error');
				}
			}
		}
		
		return $status;
	}
	
	
	protected function refreshToken() {
		$refreshToken = $this->client->getRefreshToken();
		if (!is_null($refreshToken)) {
			try {
				$this->client->setClientId($this->_clientID);
				$this->client->setClientSecret($this->_secret);
				$this->client->refreshToken($refreshToken);

				$newAccessToken = $this->client->getAccessToken();
				$this->storeAccessToken($newAccessToken);
			} catch(Exception $e) {
				if (JFactory::getApplication()->isClient('administrator')) {
					JFactory::getApplication()->enqueueMessage(JText::sprintf('RSFP_GOOGLE_SHEETS_ERROR_NO_SHEETS', $e->getMessage()), 'error');
				}
			}
		}
	}
	
	public function storeAccessToken($accessToken) {
		RSFormProConfig::getInstance()->set('google.sheets.accesstoken', $accessToken);
	}
	
	public function isInvalidToken() {
		if ($this->access_token) {
			$invalid = $this->client->isAccessTokenExpired();
			return $invalid;
		}
		
		return true;
	}
	 
	public function retrieveAccessToken($code, $redirectUri) {
		$this->client->setClientId($this->_clientID);
		$this->client->setClientSecret($this->_secret);
		$this->client->addScope(array(Google_Service_Sheets::SPREADSHEETS));
		$this->client->setRedirectUri($redirectUri);
		$this->client->setAccessType("offline");
		$this->client->authenticate($code);
		
		return $this->client->getAccessToken();
	}
	
	public static function getService($client) {
		static $service;
		if (is_null($service)) {
			$service = new Google_Service_Sheets($client);
		}
		
		return $service;
	}
	
	public function getSpreadSheetSheets($spreadsheet_id = '', $form_id = null) {
		// check that the spreadsheet id is not empty
		if (empty($spreadsheet_id)) {
			return false;
		}

		// return only if the token is still valid
		if (!$this->isInvalidToken()) {
			try {
				$service = self::getService($this->client);

				$response = $service->spreadsheets->get($spreadsheet_id);
				$sheets = $response->getSheets();

				$values = array();
				foreach ($sheets as $sheet) {
					$sheetProperties = $sheet->getProperties();
					$values[$sheetProperties->title] = RSFormProHelper::htmlEscape($sheetProperties->title);
				}

				return empty($values) ? false : $values;
			} catch (Exception $e) {
				if (!is_null($form_id)) {
					$dataLog = new stdClass();
					$dataLog->form_id = $form_id;
					$dataLog->message = $e->getMessage();
					$this->storeErrorLog($dataLog);
				}
			}
		}

		return false;
	}

	public function getSpreadSheetColumns($spreadsheet_id = '', $sheet_id = '', $form_id = null) {
		// check that the spreadsheet id is not empty
		if (empty($spreadsheet_id)) {
			return false;
		}

		// return only if the token is still valid
		if (!$this->isInvalidToken()) {
			try {
				$service = self::getService($this->client);

				$range = (!empty($sheet_id) ? $sheet_id . '!' : '') . '1:1';

				$response = $service->spreadsheets_values->get($spreadsheet_id, $range);
				$values = $response->getValues();

				return $values;
			} catch (Exception $e) {
				if (!is_null($form_id)) {
					$dataLog = new stdClass();
					$dataLog->form_id = $form_id;
					$dataLog->message = $e->getMessage();
					$this->storeErrorLog($dataLog);
				}
			}
		}

		return false;
	}

	public function addDataToSpreadSheet($spreadsheet_id, $sheet_id = '', $data = array())
	{
		$return = new stdClass();

		try
		{
			// check to see if any data is passed
			if (empty($data))
			{
				throw new Exception(JText::_('RSFP_GOOGLE_SHEETS_ERROR_NO_DATA_VALUES_PASSED_TO_THE_SPREADSHEET'));
			}

			// check that the spreadsheet id is not empty
			if (empty($spreadsheet_id))
			{
				throw new Exception(JText::_('RSFP_GOOGLE_SHEETS_ERROR_NO_SPREADSHEET_ID_SPECIFIED'));
			}

			// check if the token is still valid
			if ($this->isInvalidToken())
			{
				throw new Exception(JText::_('RSFP_GOOGLE_SHEETS_ERROR_INVALID_ACCESS_TOKEN'));
			}

			$service 	= self::getService($this->client);
			$range 		= (!empty($sheet_id) ? $sheet_id.'!' : '').'A1';

			// clean the data if the value has a <br /> in it, used mostly in multiple uploads and textareas
			foreach ($data as $colname => $value)
			{
				$value = str_replace("\r\n", "\n", $value);
				// for textareas
				$value = str_replace(array("\n <br />", "\n<br />", "<br />\n", "<br /> \n"), "\n", $value);
				// for file uploads
				$value = str_replace('<br />', "\n", $value);
				$data[$colname] = $value;
			}

			// the keys must be numbers
			$data = array_values($data);

			$body = new Google_Service_Sheets_ValueRange(array(
				'range' => $range,
				'majorDimension' => 'ROWS',
				'values' => array(
					$data
				)
			));

			$params = array(
				'valueInputOption' => 'USER_ENTERED',
				'insertDataOption' => 'INSERT_ROWS'
			);

			$result = $service->spreadsheets_values->append($spreadsheet_id, $range, $body, $params);

			if (!$result->getUpdates()->updatedRows)
			{
				throw new Exception(JText::_('RSFP_GOOGLE_SHEETS_ERROR_NO_ROWS_WERE_ADDED_TO_THE_SPREADSHEET'));
			}

			// All done
			$return->status = true;
		}
		catch (Exception $e)
		{
			$return->status = false;
			$return->msg 	= $e->getMessage();
			$return->code 	= $e->getCode();
		}

		return $return;
	}

	
	public function storeErrorLog($dataLog) {
		$dataLog->date = JFactory::getDate()->toSql();

		if (!isset($dataLog->code))
		{
			$dataLog->code = '';
		}

		JFactory::getDbo()->insertObject('#__rsform_google_sheets_error_log', $dataLog);

		$this->deleteOldErrorLog($dataLog->form_id);
	}

	protected function deleteOldErrorLog($form_id) {
		$db 	= JFactory::getDbo();
		$query 	= $db->getQuery(true);

		// get the 15th one
		$query->select('*')
			->from($db->qn('#__rsform_google_sheets_error_log'))
			->where($db->qn('form_id').' = '.$db->q($form_id))
			->order($db->qn('date').' DESC');
		$db->setQuery($query, 15, 1);

		if ($resource = $db->loadAssoc()) {
			$query = $db->getQuery(true);
			$query->delete($db->qn('#__rsform_google_sheets_error_log'))
				->where($db->qn('date').' < '.$db->q($resource['date']))
				->where($db->qn('form_id').' = '.$db->q($form_id));
			$db->setQuery($query);
			$db->execute();
		}

	}
}