<?php
/**
 * @package RSForm! Pro
 * @copyright (C) 2007-2022 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

/**
 * RSForm! Pro RSFinder! Plugin.
 */
class plgRsfinderRsform extends CMSPlugin
{
    const EXTENSION = 'com_rsform';
	
	 /**
     * @var bool
     */
    protected $autoloadLanguage = true;

    protected function canRun() {
        // Check if the component exists and is enabled - in backend
        return (
            file_exists(JPATH_ADMINISTRATOR . '/components/'.self::EXTENSION.'/helpers/rsform.php') &&
            Factory::getApplication()->isClient('administrator') &&
            ComponentHelper::getComponent(self::EXTENSION, true)->enabled
        );
    }

    public function onRsfinderGetResults(&$tmp_results, $search, $limit, &$index)
    {
        if (!$this->canRun())
        {
            return;
        }

        // verify the index first, no need to continue if the limit is reached
        if ($index >= $limit)
        {
            return;
        }

        // Get the current User
        $user = Factory::getApplication()->getIdentity();
        $access  = $user->authorise('core.manage', self::EXTENSION) && $user->authorise('core.edit', self::EXTENSION);

        // if there is no access, skip the results
        if (!$access)
        {
            return;
        }

        $select_in = $this->params->get('searchList');

        // workaround for stored version
        if (is_string($select_in))
        {
            $select_in = explode(',', $select_in);
        }

        $output_results = array('dynamic' => array(), 'fixed' => array());

        // Get DBO.
        $db	= Factory::getContainer()->get('DatabaseDriver');
		
        if (!empty($select_in))
        {

            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rsform.FormId', 'value'),
                    $db->qn('rsform.FormName', 'text'),
                ]
            )
                ->from($db->qn('#__rsform_forms', 'rsform'))
                ->where($db->qn('rsform.FormName') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

			if (in_array('forms', $select_in))
			{
				if (!empty($results))
                {
					foreach($results as $result)
                    {
                        // verify the index first
                        if ($index >= $limit)
                        {
                            break;
                        }

                        $output_result = array(
                            'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSFORM_EDIT_FORM'),
                            'link'  => Uri::root().'administrator/index.php?option=com_rsform&view=forms&layout=edit&formId='.$result->value,
                            'hits'  => 0
                        );

                        $output_results['dynamic'][] = $output_result;
						$index ++;
					}
				}
			}
			
			if ($index < $limit && in_array('submissions', $select_in))
			{
				foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSFORM_LIST_SUBMISSIONS'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsform&view=submissions&formId='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
			}
			
			if ($index < $limit && in_array('directories', $select_in))
			{
				foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSFORM_LIST_DIRECTORIES'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsform&view=directory&layout=edit&formId='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
			}
        }

        if ($index < $limit && $this->params->get('configuration', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSFORM_CONFIGURATION_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsform&view=configuration',
                'hits'  => 0
            );
        }

        if ($index < $limit && $this->params->get('backup', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSFORM_BACKUP_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsform&view=backupscreen',
                'hits'  => 0
            );
        }
		
		if ($index < $limit && $this->params->get('restore', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSFORM_RESTORE_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsform&view=restorescreen',
                'hits'  => 0
            );
        }


        $tmp_results[] = $output_results;
    }

    protected function escape($string) {
        $string = (string) $string;
        return htmlentities($string, ENT_COMPAT, 'utf-8');
    }
}