<?php
/**
* @package RSform!Pro
* @copyright (C) www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

class plgSystemRsfpgooglesheets extends JPlugin
{
	protected $autoloadLanguage = true;

	public function __construct(&$subject, $config)
	{
		parent::__construct( $subject, $config );

		if (file_exists(JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/google_sheets_class.php'))
		{
			require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/google_sheets_class.php';
		}
	}

	public function onRsformFormSave($form)
	{
		$app                    = JFactory::getApplication();
		$data                   = $app->input->post->get('gs', array(), 'array');
		$data['gds_mappings']   = array();
		$data['form_id']        = $form->FormId;
		$row                    = $this->getTable();
		if (!$row)
		{
			return false;
		}
		if ($mappings = $app->input->post->get('gds_mappings', array(), 'array'))
		{
			foreach ($mappings as $col_name => $value)
			{
				$data['gds_mappings'][base64_decode($col_name)] = $value;
			}
		}

		if (!$row->save($data))
		{
			$app->enqueueMessage($row->getError(), 'error');
		}
	}

	protected function getTable()
	{
		return JTable::getInstance('RSForm_Google_Sheets', 'Table');
	}
	
	public function onRsformBackendFormCopy($args)
	{
		$formId 	= $args['formId'];
		$newFormId 	= $args['newFormId'];

		/* @var $row TableRSForm_Google_Sheets */
		if ($row = $this->getTable())
		{
			if ($row->load($formId))
			{
				$row->form_id = $newFormId;
				if ($row->check())
				{
					$row->store();
				}
			}
		}
	}
	
	public function onRsformFormDelete($formId)
	{
		$db	    = JFactory::getDbo();
		$query 	= $db->getQuery(true);

		if ($row = $this->getTable())
		{
			$row->delete($formId);
		}

		// remove the error log for this specific form
		$query->delete('#__rsform_google_sheets_error_log')
			->where($db->qn('form_id').'='.$db->q($formId));
		$db->setQuery($query)->execute();
	}

	public function onRsformBackendAfterShowConfigurationTabs($tabs)
	{
		$tabs->addTitle(JText::_('RSFP_GOOGLE_SHEETS_LABEL'), 'page-googlesheets');
		$tabs->addContent($this->configurationScreen());
	}
	
	public function onRsformBackendAfterShowFormEditTabsTab()
	{
		?>
		<li><a href="javascript: void(0);" id="googlesheets"><span class="rsficon rsficon-google-drive"></span><span class="inner-text"><?php echo JText::_('RSFP_GOOGLE_SHEETS_SETTINGS'); ?></span></a></li>
		<?php
	}
	
	public function onRsformBackendAfterShowFormEditTabs()
	{
		$formId = JFactory::getApplication()->input->getInt('formId');
		$db 	= JFactory::getDbo();
		$row 	= $this->getTable();

		if (!$row)
		{
			return;
		}

		$row->load($formId);

		// Fields
		$lists = array();
		// Boolean inputs
		$gd_published 	= empty($row->gds_published) ? 0 : 1;
		$gd_debug 	    = empty($row->gds_debug) ? 0 : 1;
		$lists['debug']              = RSFormProHelper::renderHTML('select.booleanlist', 'gs[gds_debug]', '', $gd_debug);
		$lists['published']          = RSFormProHelper::renderHTML('select.booleanlist', 'gs[gds_published]', '', $gd_published);
		$lists['trigger_on_payment'] = RSFormProHelper::renderHTML('select.booleanlist', 'gs[gds_trigger_on_payment]', '', $row->gds_trigger_on_payment);

		$hasPayment = JPluginHelper::isEnabled('system', 'rsfppayment');

		// check if the credentials are present
		$client_id 	= RSFormProHelper::getConfig('google.sheets.clientid');
		$secret 	= RSFormProHelper::getConfig('google.sheets.secret');
		$api 		= RSFormGoogleSheets::getInstance();
		$logColumns = true;

		// Do not show these errors always, only when the "use" is set to yes
		if ($gd_published)
		{
			// check the credentials if they are present
			if (empty($client_id) || empty($secret))
			{
				JFactory::getApplication()->enqueueMessage(JText::_('RSFP_GOOGLE_SHEETS_ERROR_NO_CREDENTIALS'), 'error');
				$logColumns = false;
			}
			else
			{
				// if the credentials are present then check the access token
				if (!$api->getAccessToken())
				{
					JFactory::getApplication()->enqueueMessage(JText::_('RSFP_GOOGLE_SHEETS_ERROR_NO_ACCESSTOKEN_GENERATED'), 'error');
					$logColumns = false;
				}
			}
		}

		$columns 	= false;
		$mappings 	= array();
		$sheetsIds  = false;

		if (!empty($row->gds_spreadsheet_id))
		{
			$columns = $api->getSpreadSheetColumns($row->gds_spreadsheet_id, $row->gds_sheet_id, ($logColumns ? $formId : null));
			$sheetsIds = $api->getSpreadSheetSheets($row->gds_spreadsheet_id, ($logColumns ? $formId : null));
		}

		if (!empty($row->gds_mappings))
		{
			$mappings = unserialize($row->gds_mappings);
		}
		
		$query 	= $db->getQuery(true);
		$query->select($db->qn('code'))
			  ->select($db->qn('message'))
			  ->select($db->qn('date'))
			  ->from($db->qn('#__rsform_google_sheets_error_log'))
			  ->where($db->qn('form_id') . ' = ' . $db->q($formId))
			  ->order($db->qn('date') . ' ' . $db->escape('DESC'));
		$db->setQuery($query, 0, 15);
		$errors = $db->loadObjectList();
		?>
		<div id="googlesheetsdiv">
			<div>
				<div>
					<fieldset>
						<h3 class="rsfp-legend"><?php echo JText::_('RSFP_GOOGLE_SHEETS_SETTINGS_DESC'); ?></h3>
						<table width="100%" class="table table-bordered">
							<tr>
								<td width="80" align="right" nowrap="nowrap" class="key">
									<span class="hasTooltip" title="<?php echo JText::_('RSFP_GOOGLE_SHEETS_USE_DESC'); ?>"><?php echo JText::_('RSFP_GOOGLE_SHEETS_USE'); ?></span>
								</td>
								<td nowrap="nowrap" colspan="2"><?php echo $lists['published']; ?></td>
							</tr>
                            <?php if ($hasPayment) { ?>
                            <tr>
                                <td width="80" align="right" nowrap="nowrap" class="key">
                                    <span class="hasTooltip" title="<?php echo JText::_('RSFP_GOOGLE_SHEETS_TRIGGER_ON_PAYMENT_DESC'); ?>"><?php echo JText::_('RSFP_GOOGLE_SHEETS_TRIGGER_ON_PAYMENT'); ?></span>
                                </td>
                                <td nowrap="nowrap" colspan="2"><?php echo $lists['trigger_on_payment']; ?></td>
                            </tr>
                            <?php } ?>
							<tr>
								<td width="80" align="right" nowrap="nowrap" class="key">
									<span class="hasTooltip" title="<?php echo JText::_('RSFP_GOOGLE_SHEETS_DEBUG_DESC'); ?>"><?php echo JText::_('RSFP_GOOGLE_SHEETS_DEBUG'); ?></span>
								</td>
								<td nowrap="nowrap" colspan="2"><?php echo $lists['debug']; ?></td>
							</tr>
							<tr>
								<td width="80" align="right" nowrap="nowrap" class="key">
									<span class="hasTooltip" title="<?php echo JText::_('RSFP_GOOGLE_SHEETS_ID_DESC'); ?>"><?php echo JText::_('RSFP_GOOGLE_SHEETS_ID'); ?></span>
								</td>
								<td nowrap="nowrap" colspan="2"><input type="text" id="gds_spreadsheet_id" name="gs[gds_spreadsheet_id]"  value="<?php echo RSFormProHelper::htmlEscape($row->gds_spreadsheet_id); ?>"/></td>
							</tr>
							<tr>
								<td colspan="2">
									<small><?php echo JText::_('RSFP_GOOGLE_SHEETS_ID_LONG_DESC'); ?></small>
									<div class="alert alert-info">
										<?php echo JText::_('RSFP_GOOGLE_SHEETS_SUBMIT_DESC'); ?>
									</div>
								</td>
							</tr>
							<?php if ($sheetsIds) { ?>
							<tr>
								<td width="80" align="right" nowrap="nowrap" class="key">
									<span class="hasTooltip" title="<?php echo JText::_('RSFP_GOOGLE_SHEETS_SHEET_ID_DESC'); ?>"><?php echo JText::_('RSFP_GOOGLE_SHEETS_SHEET_ID'); ?></span>
								</td>
								<td nowrap="nowrap" colspan="2">
									<?php echo JHtml::_('select.genericlist', $sheetsIds, 'gs[gds_sheet_id]', 'onchange="rsf_google_sheets_columns()"', 'value', 'text', isset($row->gds_sheet_id) ? $row->gds_sheet_id : null); ?>
								</td>
							</tr>
							<tr>
								<td colspan="2">
									<small><?php echo JText::_('RSFP_GOOGLE_SHEETS_SHEET_ID_LONG_DESC'); ?></small>
								</td>
							</tr>
							<?php } ?>
						</table>
					</fieldset>
					
					<div id="rsf_google_sheets_fields">
					<?php
					if ($columns) {
						$columns = $columns[0];
						?>
						<table class="table table-bordered" <?php echo !$lists['published'] ? 'style="display: none;"' : ''; ?>>
							<?php foreach($columns as $i => $col) {
								?>
								<tr>
									<td width="80" align="right" nowrap="nowrap" class="key"><?php echo RSFormProHelper::htmlEscape($col); ?></td>
									<td>
										<input name="gds_mappings[<?php echo base64_encode($col); ?>]" style="width: 80%;" id="gds_mappings_<?php echo $i; ?>"  value="<?php echo (isset($mappings[$col]) ? RSFormProHelper::htmlEscape($mappings[$col]) : ''); ?>" class="rs_inp rs_80"  data-delimiter=" " data-placeholders="display" />
									</td>
								</tr>
							<?php } ?>
						</table>
					<?php } else if (!$columns && $logColumns) { ?>
						<div class="alert alert-error">
							<?php echo JText::_('RSFP_GOOGLE_SHEETS_ERROR_NO_COLUMNS_DEFINED'); ?>
						</div>
					<?php } ?>
					</div>
				</div>
				<div>
					<fieldset>
						<legend><?php echo JText::_('RSFP_GOOGLE_SHEETS_FORM_ERROR_LOG'); ?></legend>
						<?php if (empty($errors)) { ?>
							<div class="alert alert-error">
								<?php echo JText::_('RSFP_GOOGLE_SHEETS_NO_ERRORS_SO_FAR'); ?>
							</div>
						<?php } else { ?>
							<table class="table table-responsive table-condensed table-bordered">
								<thead>
								<tr>
									<th><?php echo JText::_('RSFP_GOOGLE_SHEETS_CODE'); ?></th>
									<th><?php echo JText::_('RSFP_GOOGLE_SHEETS_MESSAGE'); ?></th>
									<th><?php echo JText::_('RSFP_GOOGLE_SHEETS_DATETIME'); ?></th>
								</tr>
								</thead>
								<?php foreach ($errors as $error) { ?>
									<tr>
										<td><?php echo (empty($error->code) ? ' -- ' : $error->code); ?></td>
										<td><?php echo RSFormProHelper::htmlEscape($error->message); ?></td>
										<td><?php echo RSFormProHelper::getDate($error->date); ?></td>
									</tr>
								<?php } ?>
							</table>
						<?php } ?>
					</fieldset>
				</div>
			</div>
			<script type="text/javascript">
				function rsf_google_sheets_columns() {
					var spreadsheet_id = jQuery('[name="gs[gds_spreadsheet_id]"]').val();
					var sheet_id = jQuery('[name="gs[gds_sheet_id]"]').val();

					if (spreadsheet_id.trim() == '') {
						return;
					}

					jQuery.ajax({
						url: 'index.php?option=com_rsform&task=plugin&plugin_task=get_google_sheets_columns',
						method: "POST",
						data: {spreadsheet_id: spreadsheet_id, sheet_id: sheet_id, form_id: <?php echo $formId; ?>},
						dataType: "html"
					}).done(function (response) {
						var container = jQuery('#rsf_google_sheets_fields');
						container.html(response);
						container.find('[data-placeholders]').rsplaceholder();
					});
				}
			</script>
		</div>
		<?php
	}
	
	private function configurationScreen()
	{
		ob_start();

		JForm::addFormPath(__DIR__ . '/forms');

		$form = JForm::getInstance( 'plg_system_rsfpgooglesheets.configuration', 'configuration', array('control' => 'rsformConfig'), false, false );
		$form->bind($this->loadFormData());

		$clientId   = $form->getValue('google.sheets.clientid');
		$secret     = $form->getValue('google.sheets.secret');
		?>
		<div id="page-googlesheets" class="form-horizontal">
			<?php
			foreach ($form->getFieldsets() as $fieldset)
			{
				if ($fields = $form->getFieldset($fieldset->name))
				{
					foreach ($fields as $field)
					{
						echo $field->renderField();
					}
				}
			}
			?>
			<div class="alert alert-info">
				<h4><?php echo JText::_('RSFP_GOOGLE_SHEETS_REDIRECT_URI'); ?></h4>
				<p><?php echo JText::_('RSFP_GOOGLE_SHEETS_REDIRECT_URI_DESC'); ?></p>
				<p><strong><?php echo JUri::root().'administrator/index.php?option=com_rsform&task=plugin&plugin_task=store_google_sheets_access_token'; ?></strong></p>
			</div>
			<?php
			if (!empty($clientId) && !empty($secret))
			{
				$api 			= RSFormGoogleSheets::getInstance();
				$accesstoken 	= $api->getAccessToken();

				$status = $this->accessTokenStatus($api, $accesstoken);
				if ($status)
				{
					foreach ($status as $key => $value)
					{
						?>
						<div class="control-group">
							<div class="control-label">
								<label class="hasTooltip" title="<?php echo JText::_('RSFP_GOOGLE_SHEETS_'.strtoupper($key).'_DESC'); ?>"><?php echo JText::_('RSFP_GOOGLE_SHEETS_'.strtoupper($key)); ?></label>
							</div>
							<div class="controls">
								<?php
								if (is_bool($value))
								{
									echo $value ? JText::_('RSFP_GOOGLE_SHEETS_EXPIRED') : JText::_('RSFP_GOOGLE_SHEETS_VALID');
								}
								else
								{
									echo $value;
								}
								?>
							</div>
						</div>
						<?php
					}

					if ($status->is_invalid)
					{
						?>
						<div class="control-group">
							<div class="control-label">
								<label class="hasTooltip" title="<?php echo JText::_('RSFP_GOOGLE_SHEETS_RENEW_ACCESS_TOKEN_DESC'); ?>"><?php echo JText::_('RSFP_GOOGLE_SHEETS_RENEW_ACCESS_TOKEN'); ?></label>
							</div>
							<div class="controls">
								<a href="<?php echo $api->getAuthURL(JUri::root().'administrator/index.php?option=com_rsform&task=plugin&plugin_task=store_google_sheets_access_token'); ?>" class="btn btn-success"><?php echo JText::_('RSFP_GOOGLE_SHEETS_RENEW'); ?></a>
							</div>
						</div>
						<?php
					}
					else
					{
						?>
						<div class="control-group">
							<div class="control-label">
								<label class="hasTooltip" title="<?php echo JText::_('RSFP_GOOGLE_SHEETS_REVOKE_ACCESS_TOKEN_DESC'); ?>"><?php echo JText::_('RSFP_GOOGLE_SHEETS_REVOKE_ACCESS_TOKEN'); ?></label>
							</div>
							<div class="controls">
								<a href="<?php echo JUri::root().'administrator/index.php?option=com_rsform&task=plugin&plugin_task=revoke_google_sheets_access_token'; ?>" class="btn btn-danger"><?php echo JText::_('RSFP_GOOGLE_SHEETS_REVOKE'); ?></a>
							</div>
						</div>
						<?php
					}
				}
				else
				{
					?>
					<div class="control-group">
						<div class="control-label">
							<label class="hasTooltip" title="<?php echo JText::_('RSFP_GOOGLE_SHEETS_GET_ACCESS_TOKEN_DESC'); ?>"><?php echo JText::_('RSFP_GOOGLE_SHEETS_GET_ACCESS_TOKEN'); ?></label>
						</div>
						<div class="controls">
							<a href="<?php echo $api->getAuthURL(JUri::root().'administrator/index.php?option=com_rsform&task=plugin&plugin_task=store_google_sheets_access_token'); ?>" class="btn btn-primary"><?php echo JText::_('RSFP_GOOGLE_SHEETS_AUTHENTICATE'); ?></a>
						</div>
					</div>
					<?php
				}
			}
			?>
		</div>
		<?php

		$contents = ob_get_contents();
		ob_end_clean();

		return $contents;
	}

	private function loadFormData()
	{
		$data 	= array();
		$db 	= JFactory::getDbo();

		$query = $db->getQuery(true)
			->select('*')
			->from($db->qn('#__rsform_config'))
			->where($db->qn('SettingName') . ' LIKE ' . $db->q('google.sheets.%', false));
		if ($results = $db->setQuery($query)->loadObjectList())
		{
			foreach ($results as $result)
			{
				$data[$result->SettingName] = $result->SettingValue;
			}
		}

		return $data;
	}
	
	public function onRsformBackendSwitchTasks()
	{
		$app 			= JFactory::getApplication();
		$input 			= $app->input;
		$plugin_task 	= $input->getCmd('plugin_task','');
		
		switch ($plugin_task)
		{
			// store the access token that we recive from the google OAuth
			case 'store_google_sheets_access_token':
				$code 			= $input->getString('code', '');
				$redirectUri 	= JUri::root().'administrator/index.php?option=com_rsform&task=plugin&plugin_task=store_google_sheets_access_token';

				try {
					$api 			= RSFormGoogleSheets::getInstance();
					$accessToken 	= $api->retrieveAccessToken($code, $redirectUri);

					$api->storeAccessToken($accessToken);
					$app->enqueueMessage(JText::_('RSFP_GOOGLE_SHEETS_STORED_ACCESS_TOKEN_SUCCESSFUL'), 'success');
				} catch (Exception $e) {
					$app->enqueueMessage(JText::sprintf('RSFP_GOOGLE_SHEETS_ERROR_NO_SHEETS', $e->getMessage()), 'error');
				}

				$app->redirect(JRoute::_('index.php?option=com_rsform&view=configuration', false));
			break;
			
			// revoke the access token that exists
			case 'revoke_google_sheets_access_token':
				$api = RSFormGoogleSheets::getInstance();

				if ($api->revokeToken())
				{
					$app->enqueueMessage(JText::_('RSFP_GOOGLE_SHEETS_REVOKE_ACCESS_TOKEN_SUCCESSFUL'), 'success');
				}
				else
				{
					$app->enqueueMessage(JText::_('RSFP_GOOGLE_SHEETS_REVOKE_ACCESS_TOKEN_UNSUCCESSFUL'), 'error');
				}
				
				$app->redirect(JRoute::_('index.php?option=com_rsform&view=configuration', false));
			break;
			
			case 'get_google_sheets_columns':
				ob_start();
				$spreadsheet_id = $input->getString('spreadsheet_id', '');
				$sheet_id = $input->getString('sheet_id', '');
				$formId = $input->getString('form_id', '');

				$api 		= RSFormGoogleSheets::getInstance();

				$columns = $api->getSpreadSheetColumns($spreadsheet_id, $sheet_id, $formId);

				$row 	= $this->getTable();

				if (!$row)
				{
					return;
				}

				$row->load($formId);

				if (!empty($row->gds_mappings))
				{
					$mappings = unserialize($row->gds_mappings);
				}

				if ($columns) {
					$columns = $columns[0];
					?>
					<table class="table table-bordered">
						<?php foreach($columns as $i => $col) {
							?>
							<tr>
								<td width="80" align="right" nowrap="nowrap" class="key"><?php echo RSFormProHelper::htmlEscape($col); ?></td>
								<td>
									<input name="gds_mappings[<?php echo base64_encode($col); ?>]" style="width: 80%;" id="gds_mappings_<?php echo $i; ?>"  value="<?php echo (isset($mappings[$col]) ? RSFormProHelper::htmlEscape($mappings[$col]) : ''); ?>" class="rs_inp rs_80"  data-delimiter=" " data-placeholders="display" />
								</td>
							</tr>
						<?php } ?>
					</table>

				<?php
				}
				else
				{
					?>
					<div class="alert alert-error">
						<?php echo JText::_('RSFP_GOOGLE_SHEETS_ERROR_NO_COLUMNS_DEFINED'); ?>
					</div>
					<?php
				}

				$contents = ob_get_contents();
				ob_end_clean();

				echo $contents;

				$app->close();
			break;
		}
	}
	
	// parse the access token and out the current data
	protected function accessTokenStatus($api, $accesstoken)
	{
		$status = false;

		if ($accesstoken)
		{
			$status = json_decode($accesstoken);
			$config = JFactory::getConfig();
			$tz 	= $config->get('offset');

			$createdDate = JFactory::getDate($status->created);
			$createdDate->setTimezone(new DateTimeZone($tz));
			$createdDate->format(RSFormProHelper::getConfig('global.date_mask'));

			$expires = $status->created + $status->expires_in;
			$expiresAt = JFactory::getDate($expires);
			$expiresAt->setTimezone(new DateTimeZone($tz));
			$expiresAt->format(RSFormProHelper::getConfig('global.date_mask'));

			$status->created = $createdDate;
			$status->expires_at = $expiresAt;
			// we do not need this anymore
			unset($status->expires_in);

			$status->is_invalid = $api->isInvalidToken();
		}
		
		return $status;
	}

	private function getSettings($formId)
    {
	    static $cache = array();

	    if (!isset($cache[$formId]))
	    {
		    $db    = JFactory::getDbo();
		    $query = $db->getQuery(true);
		    $query->select('*')
			    ->from($db->qn('#__rsform_google_sheets'))
			    ->where($db->qn('form_id') . ' = ' . $db->q($formId))
			    ->where($db->qn('gds_published') . ' = ' . $db->q(1));
		    $db->setQuery($query);

		    $cache[$formId] = $db->loadObject();
	    }

	    if (!empty($cache[$formId]))
	    {
		    return clone $cache[$formId];
	    }

	    return false;
    }

    private function updateSheet($formId, $SubmissionId, $row)
    {
	    list($replace, $with) = RSFormProHelper::getReplacements($SubmissionId);
	    $replace[] = '\n';
	    $with[] = "\n";

	    if (empty($row->gds_mappings))
	    {
		    return;
	    }

	    if (empty($row->gds_spreadsheet_id))
	    {
		    return;
	    }

	    $data = unserialize($row->gds_mappings);

	    foreach ($data as $colname => $value)
	    {
		    $data[$colname] = str_replace($replace, $with, $value);
	    }

	    $api = RSFormGoogleSheets::getInstance();

	    // if there are differences between the mapping and the actual columns in the Spreadsheet, populate only the ones that match
	    $columns = $api->getSpreadSheetColumns($row->gds_spreadsheet_id, $row->gds_sheet_id, $formId);
	    $columns = $columns[0];

	    $stored_columns = array_keys($data);
	    if ($stored_columns != $columns)
	    {
		    $new_data = array();
		    foreach ($columns as $colname)
		    {
			    if (isset($data[$colname]))
			    {
				    $new_data[$colname] = $data[$colname];
			    }
		    }

		    $data = $new_data;
	    }

	    $added = $api->addDataToSpreadSheet($row->gds_spreadsheet_id, $row->gds_sheet_id, $data);

	    if (!$added->status)
	    {
		    $dataLog = new stdClass();
		    $dataLog->form_id = $formId;
		    $dataLog->message = $added->msg;
		    if (isset($added->code))
		    {
			    $dataLog->code = $added->code;
		    }

		    // if debug is active
		    if ($row->gds_debug)
		    {
			    JFactory::getApplication()->enqueueMessage(JText::sprintf('RSFP_GOOGLE_SHEETS_ERROR_NO_SHEETS', $dataLog->message), 'warning');
		    }

		    $api->storeErrorLog($dataLog);
	    }
    }

	public function onRsformAfterConfirmPayment($SubmissionId)
	{
		require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/submissions.php';
		$submission = RSFormProSubmissionsHelper::getSubmission($SubmissionId, false);

		if ($submission)
		{
			if ($row = $this->getSettings($submission->FormId))
			{
				if ($row->gds_trigger_on_payment)
				{
					$this->updateSheet($submission->FormId, $SubmissionId, $row);
				}
			}
		}
	}
	
	/*
	* Functions used only inside this class
	*/
	public function onRsformFrontendAfterFormProcess($args)
	{
		$formId 		= (int) $args['formId'];
		$SubmissionId 	= (int) $args['SubmissionId'];

		if ($row = $this->getSettings($formId))
		{
		    if (empty($row->gds_trigger_on_payment))
            {
                $this->updateSheet($formId, $SubmissionId, $row);
            }
		}
	}
	
	/*
	* End Functions used only inside this class
	*/
	public function onRsformFormBackup($form, $xml, $fields)
	{
		$db 	= JFactory::getDbo();
		$query 	= $db->getQuery(true);

		if ($row = $this->getTable())
		{
			if ($row->load($form->FormId))
			{
				$row->check();

				$data = $row->getProperties();
				unset($data['form_id']);

				$xml->add('googlesheets');
				foreach ($data as $property => $value)
				{
					$xml->add($property, $value);
				}
				$xml->add('/googlesheets');
			}
		}
		
		// export the error log
		$query->select('*')
			  ->from($db->qn('#__rsform_google_sheets_error_log'))
			  ->where($db->qn('form_id').'='.$db->q($form->FormId));
		$db->setQuery($query);
		if ($errors = $db->loadObjectList())
		{
			$xml->add('googlesheetslog');
			foreach ($errors as $error)
			{
				// No need for a form_id
				unset($error->form_id);
				unset($error->id);
				
				$xml->add('error');
				foreach ($error as $property => $value)
				{
					$xml->add($property, $value);
				}
				$xml->add('/error');
			}
			$xml->add('/googlesheetslog');
		}
	}
	
	public function onRsformFormRestore($form, $xml, $fields)
	{
		if (isset($xml->googlesheets))
		{
			$data = array(
				'form_id' => $form->FormId
			);
			
			foreach ($xml->googlesheets->children() as $property => $value)
			{
				$data[$property] = (string) $value;
			}
			
			$row = $this->getTable();
			$row->save($data);
		}
		
		// the error log
		if (isset($xml->googlesheetslog))
		{
			$db = JFactory::getDbo();

			foreach ($xml->googlesheetslog->error  as $error)
			{
				$data = array(
					$db->qn('form_id') .'='. $db->q($form->FormId)
				);
				
				foreach ($error->children() as $property => $value)
				{
					$value = (string) $value;
					$data[] = $db->qn($property) .'='. $db->q($value);
				}

				$query = $db->getQuery(true);
				$query	->insert('#__rsform_google_sheets_error_log')
						->set($data);
				$db->setQuery($query)->execute();
			}
		}
	}
}