<?php
/**
 * @package        RSForm! Pro
 * @copyright  (c) 2007 - 2016 RSJoomla!
 * @link           https://www.rsjoomla.com
 * @license        GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;

/**
 * Class plgSystemRsfpverticalresponse
 */
class plgSystemRsfpverticalresponse extends JPlugin
{

	/**
	 * @var bool
	 */
	protected $autoloadLanguage = true;

	/**
	 * @param $form
	 *
	 * @return void
	 */
	public function onRsformFormSave($form)
	{
		$app             = JFactory::getApplication();
		$data            = $app->input->post->get('vr', array(), 'array');
		$data['vars']    = $app->input->post->get('verticalresponse_vars', array(), 'array');
		$data['form_id'] = $form->FormId;

		$row = $this->getTable();
		if ($row)
		{
			if (!$row->save($data))
			{
				$app->enqueueMessage($row->getError(), 'error');
			}
		}
	}
	
	protected function getTable()
	{
		return JTable::getInstance('RSForm_VerticalResponse', 'Table');
	}
	
	public function onRsformBackendFormCopy($args)
	{
		$formId = $args['formId'];
		$newFormId = $args['newFormId'];

		if ($row = $this->getTable())
		{
			if ($row->load($formId))
			{
				if (!$row->bind(array('form_id'=>$newFormId)))
				{
					return false;
				}

				$row->check();

				if (!$row->store())
				{
					JFactory::getApplication()->enqueueMessage($row->getError(), 'warning');
				}
			}
		}
	}

	/**
	 * @param $tabs
	 */
	public function onRsformBackendAfterShowConfigurationTabs($tabs)
	{
		$tabs->addTitle(JText::_('RSFP_VERTICALRESPONSE_LABEL'), 'page-verticalresponse');
		$tabs->addContent($this->showConfigurationScreen());
	}

	/**
	 * @return bool
	 * @throws Exception
	 */
	public function onAfterRoute()
	{
		$app = JFactory::getApplication();

		if (!$app->isClient('administrator'))
		{
			return false;
		}

		if (!JFactory::getUser()->id)
		{
			return false;
		}
		$code = $app->input->get->get('code', '');
		$referer = $app->input->server->getString('HTTP_REFERER');
		if (strlen($code) == 24 && strpos(strtolower($referer), 'https://vrapi.verticalresponse.com') === 0)
		{
			if (!class_exists('RSFormProHelper'))
			{
				$helper = JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/rsform.php';
				if (file_exists($helper))
				{
					require_once($helper);
					RSFormProHelper::readConfig();
				}
				else
				{
					return false;
				}
			}

			$http          = JHttpFactory::getHttp();
			$client_id     = RSFormProHelper::getConfig('verticalresponse.key');
			$client_secret = RSFormProHelper::getConfig('verticalresponse.secret');

			$returnUrl = urlencode(JUri::root() . 'administrator/index.php');

			$url = 'https://vrapi.verticalresponse.com/api/v1/oauth/access_token?client_id=' . $client_id . '&client_secret=' . $client_secret . '&redirect_uri=' . $returnUrl . '&code=' . $code;

			try
			{
				$response = $http->get($url, array(), 3);

				if ($response->code != 200)
				{
					throw new Exception(JText::sprintf('RSFP_VERTICALRESPONSE_CONNECTION_ERROR', $response->code));
				}

				$token = $response->body;

				RSFormProConfig::getInstance()->set('verticalresponse.token', $token);

				JFactory::getDocument()->addScriptDeclaration("
					window.opener.document.getElementsByName('rsformConfig[verticalresponse.token]')[0].value = " . json_encode($token) . ";
					window.opener.Joomla.renderMessages({'success': [" . json_encode(JText::_('RSFP_VERTICALRESPONSE_TOKEN_SAVED')) . "]});
					window.close();
				");
			}
			catch (Exception $e)
			{
				JFactory::getDocument()->addScriptDeclaration("
					window.opener.Joomla.renderMessages({'error': [" . json_encode($e->getMessage()) . "]});
					window.close();
				");
			}
			$app->setHeader('cross-origin-opener-policy', 'unsafe-none', true);
		}
	}

	private function loadFormData()
	{
		$data 	= array();
		$db 	= JFactory::getDbo();

		$query = $db->getQuery(true)
			->select('*')
			->from($db->qn('#__rsform_config'))
			->where($db->qn('SettingName') . ' LIKE ' . $db->q('verticalresponse.%', false));
		if ($results = $db->setQuery($query)->loadObjectList())
		{
			foreach ($results as $result)
			{
				$data[$result->SettingName] = $result->SettingValue;
			}
		}

		return $data;
	}

	/**
	 * @return string
	 */
	protected function showConfigurationScreen()
	{
		ob_start();

		JForm::addFormPath(__DIR__ . '/forms');
		$form = JForm::getInstance( 'plg_system_rsfpverticalresponse.configuration', 'configuration', array('control' => 'rsformConfig'), false, false );
		$form->bind($this->loadFormData());

		$key = RSFormProHelper::getConfig('verticalresponse.key');

		$url = '';
		if ($key)
		{
			$returnUrl = urlencode(JUri::root() . 'administrator/index.php');
			$url       = 'https://vrapi.verticalresponse.com/api/v1/oauth/authorize?client_id=' . $key . '&redirect_uri=' . $returnUrl;
		}

		JFactory::getApplication()->setHeader('cross-origin-opener-policy', 'unsafe-none', true);
		?>
		<div id="page-verticalresponse" class="form-horizontal">
			<?php
			foreach ($form->getFieldsets() as $fieldset)
			{
				if ($fields = $form->getFieldset($fieldset->name))
				{
					foreach ($fields as $field)
					{
						echo str_replace('"rsformConfig[verticalresponse][', '"rsformConfig[verticalresponse.', $form->renderField($field->fieldname));
					}
				}
			}
			?>
			<div>
				<?php
				$buttonClasses = array('btn');
				$buttonClasses[] = $key ? 'btn-success' : 'btn-danger';
				?>
				<button type="button" class="<?php echo implode(' ', $buttonClasses); ?>" <?php if (!$key) { ?>disabled="disabled"<?php } ?> onclick="window.open(this.getAttribute('data-url'), 'vrAuth', 'width=600,height=700,menubar=no');" data-url="<?php echo RSFormProHelper::htmlEscape($url); ?>"><?php echo JText::_('RSFP_VERTICALRESPONSE_AUTHORIZE'); ?></button>
			</div>
			<?php
			if (!$key)
			{
				?>
				<p>
					<a href="https://developer.verticalresponse.com/" class="btn btn-primary" target="_blank"><?php echo JText::_('RSFP_VERTICALRESPONSE_GETKEY'); ?></a>
				</p>
				<?php
			}
			?>
		</div>

		<?php
		$contents = ob_get_contents();
		ob_end_clean();

		return $contents;
	}

	/**
	 *
	 */
	public function onRsformBackendAfterShowFormEditTabsTab()
	{
		?>
		<li><a href="javascript: void(0);" id="rsfpverticalresponse"><span class="rsficon rsficon-envelope-square"></span><span class="inner-text"><?php echo JText::_('RSFP_VERTICALRESPONSE_LABEL'); ?></span></a></li>
		<?php
	}

	/**
	 * @throws Exception
	 */
	public function onRsformBackendAfterShowFormEditTabs()
	{
		$formId = JFactory::getApplication()->input->getInt('formId', 0);
		$row    = $this->getTable();
		$token  = RSFormProHelper::getConfig('verticalresponse.token');
		$app    = JFactory::getApplication();

		if ($token == '')
		{
			?>
			<div id="rsfpverticalresponsediv">
				<div class="alert alert-warning"><?php echo JText::_('RSFP_VERTICALRESPONSE_NOTOKEN') ?></div>
			</div>
			<?php
			return;
		}

		if (!$row)
		{
			return;
		}

		$row->load($formId);

		if ($row->vars)
		{
			$row->vars = unserialize($row->vars);
		}
		
		if (!empty($row->vars['company_name'])) {
			$row->vars['company'] = $row->vars['company_name'];
			unset($row->vars['company_name']);
		}

		$fields_array = $this->_getFields($formId);
		$fields       = array(
			JHtml::_('select.option', '', JText::_('JSELECT')),
		);
		foreach ($fields_array as $field)
		{
			$fields[] = JHtml::_('select.option', '{' . $field . ':value}', $field);
		}

		// Merge Vars
		$merge_vars = $this->standardFields();

		echo '<div id="rsfpverticalresponsediv">';

		try
		{
			require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/verticalresponse/vr_api_client.php';
			require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/verticalresponse/vr_api_contact_list.php';

			$token = json_decode($token);

			if ($token === null)
			{
				throw new Exception(JText::_('RSFP_VERTICALRESPONSE_COULD_NOT_DECODE_JSON_DATA'));
			}

			if (!defined('ROOT_URL'))
			{
				define('ROOT_URL', 'https://vrapi.verticalresponse.com/api/v1/');
			}
			if (!defined('VR_API_ACCESS_TOKEN'))
			{
				define('VR_API_ACCESS_TOKEN', $token->access_token);
			}

			$client = new VR_APIClient(null);

			$response = $client->get(ROOT_URL . 'custom_fields');

			if ($response == 'Invalid or expired token')
			{
				throw new Exception(JText::_('RSFP_VERTICALRESPONSE_INVALIDTOKEN'));
			}

			if (!empty($response['items']))
			{
				foreach ($response['items'] as $custom_field)
				{
					$merge_vars[$custom_field['attributes']['name']] = $custom_field['attributes']['name'];
				}
			}
?>
	<table class="table table-bordered">
		<tr>
			<td width="80" align="right" nowrap="nowrap" class="key">
				<label for="enable_verticalresponse"><?php echo JText::_('RSFP_VERTICALRESPONSE_ENABLE'); ?></label>
			</td>
			<td>
				<?php echo RSFormProHelper::renderHTML('select.booleanlist', 'vr[enable_verticalresponse]', '', $row->enable_verticalresponse); ?>
			</td>
		</tr>
		<tr class="vertical_response_fields">
			<td width="80" align="right" nowrap="nowrap" class="key">
				<label for="verticalresponse_update"><?php echo JText::_('RSFP_VERTICALRESPONSE_UPDATE_IF_EXISTS'); ?></label>
			</td>
			<td>
				<?php echo RSFormProHelper::renderHTML('select.booleanlist', 'vr[verticalresponse_update]', '', $row->verticalresponse_update); ?>
			</td>
		</tr>

		<tr class="vertical_response_fields">
			<td width="80" align="right" nowrap="nowrap" class="key">
				<label for="verticalresponse_list"><?php echo JText::_('RSFP_VERTICALRESPONSE_LISTS'); ?></label>
			</td>
			<td>
				<?php
				$response = contactList::get(ROOT_URL . 'lists', array('type' => 'basic'));

				$lists = array();
				foreach ($response['items'] as $item)
				{
					$lists[] = JHtml::_('select.option', $item['attributes']['id'], $item['attributes']['name']);
				}

				?>
				<select name="vr[verticalresponse_list]" id="verticalresponse_list">
					<?php echo JHtml::_('select.options', $lists, 'value', 'text', RSFormProHelper::htmlEscape($row->verticalresponse_list)); ?>
				</select>

			</td>
		</tr>
		<?php
		foreach ($merge_vars as $merge_var => $title)
		{ ?>
			<tr class="vertical_response_fields">
				<td width="80" class="key" nowrap="nowrap" align="right">
					<label for="<?php echo 'verticalresponse_vars' . $merge_var ?>"><?php echo $title; ?></label>
				</td>
				<td>
					<?php echo JHtml::_('select.genericlist', $fields, 'verticalresponse_vars[' . $merge_var . ']', null, 'value', 'text', isset($row->vars[$merge_var]) ? $row->vars[$merge_var] : null); ?>
				</td>
			</tr>
		<?php } ?>

	</table>
<?php
		}
		catch (Exception $e)
		{
			$app->enqueueMessage($e->getMessage(), 'warning');
		}

		echo '</div>';
	}

	/**
	 * @param $args
	 */
	public function onRsformFrontendAfterFormProcess($args)
	{
		$db           = JFactory::getDbo();
		$formId       = (int) $args['formId'];
		$SubmissionId = (int) $args['SubmissionId'];

		$query = $db->getQuery(true)
			->select('*')
			->from($db->qn('#__rsform_verticalresponse'))
			->where($db->qn('form_id') . '=' . $db->q($formId))
			->where($db->qn('enable_verticalresponse') . '=' . $db->q(1));
		$db->setQuery($query);

		if ($row = $db->loadObject())
		{
			try
			{
				require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/verticalresponse/vr_api_contact.php';
				require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/verticalresponse/vr_api_contact_list.php';

				$token = RSFormProHelper::getConfig('verticalresponse.token');

				if (!$token)
				{
					return true;
				}

				$token = json_decode($token);

				if ($token === null)
				{
					throw new Exception(JText::_('RSFP_VERTICALRESPONSE_COULD_NOT_DECODE_JSON_DATA'));
				}

				if (!defined('ROOT_URL'))
				{
					define('ROOT_URL', 'https://vrapi.verticalresponse.com/api/v1/');
				}
				if (!defined('VR_API_ACCESS_TOKEN'))
				{
					define('VR_API_ACCESS_TOKEN', $token->access_token);
				}

				// Get replacements
				list($args['placeholders'], $args['values']) = RSFormProHelper::getReplacements($SubmissionId);

				$standard_fields = $this->standardFields();

				// Grab configured fields
				$fields = array();
				if ($row->vars)
				{
					$fields = unserialize($row->vars);
				}

				if (!is_array($fields))
				{
					$fields = array();
				}

				$element = array();
				foreach ($fields as $field => $value)
				{
					// Custom fields should be in a separate array
					if (array_key_exists($field, $standard_fields))
					{
						$element[$field] = str_replace($args['placeholders'], $args['values'], $value);
					}
					else
					{
						$element['custom'][$field] = str_replace($args['placeholders'], $args['values'], $value);
					}
				}

				$element = array_filter($element);

				if (!isset($element['email']))
				{
					throw new Exception(JText::_('RSFP_VERTICALRESPONSE_NOEMAIL_MAPPED'));
				}

				if ($row->verticalresponse_list !== '')
				{
					$url = ROOT_URL . 'lists/' . $row->verticalresponse_list . '/contacts';

				}

				else
				{
					$url = ROOT_URL . 'contacts';
				}

				if ($row->verticalresponse_update)
				{
					$search = $this->userExists($element);

					if ($search)
					{
						$url = ROOT_URL . 'contacts/' . $search;

						unset($element['email']);
						$result = VR_APIClient::put($url, $element);
					}
					else
					{
						$result = VR_APIClient::post($url, $element);
					}

				}
				else
				{
					$result = VR_APIClient::post($url, $element);
				}

				if ($result == 'Invalid or expired token')
				{
					throw new Exception(JText::_('RSFP_VERTICALRESPONSE_INVALIDTOKEN'));
				}

				if (!empty($result['code']) && $result['code'] != 200 && !empty($result['message']))
				{
					throw new Exception($result['message']);
				}
			}
			catch (Exception $e)
			{
				JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
			}
		}

	}

	/**
	 * @param $formId
	 *
	 * @return mixed
	 */
	protected function _getFields($formId)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true)
			->select($db->qn('p.PropertyValue'))
			->from($db->qn('#__rsform_components', 'c'))
			->join('LEFT', $db->qn('#__rsform_properties', 'p') . ' ON (' . $db->qn('c.ComponentId') . '=' . $db->qn('p.ComponentId') . ')')
			->where($db->qn('c.FormId') . '=' . $db->q($formId))
			->where($db->qn('p.PropertyName') . '=' . $db->q('NAME'))
			->order($db->qn('c.Order') . ' ' . $db->escape('ASC'));

		return $db->setQuery($query)->loadColumn();
	}

	/**
	 * @return array
	 */
	public function standardFields()
	{
		return array(
			'email'               => JText::_('RSFP_VERTICALRESPONSE_EMAIL'),
			'first_name'          => JText::_('RSFP_VERTICALRESPONSE_FIRSTNAME'),
			'last_name'           => JText::_('RSFP_VERTICALRESPONSE_LASTNAME'),
			'marital_status'      => JText::_('RSFP_VERTICALRESPONSE_MARITAL_STATUS'),
			'birthdate_month'     => JText::_('RSFP_VERTICALRESPONSE_BIRTHMONTH'),
			'birthdate_day'       => JText::_('RSFP_VERTICALRESPONSE_BIRTHDAY'),
			'birthdate_year'      => JText::_('RSFP_VERTICALRESPONSE_BIRTHYEAR'),
			'gender'              => JText::_('RSFP_VERTICALRESPONSE_GENDER'),
			'work_phone'          => JText::_('RSFP_VERTICALRESPONSE_WORK_PHONE'),
			'home_phone'          => JText::_('RSFP_VERTICALRESPONSE_HOME_PHONE'),
			'mobile_phone'        => JText::_('RSFP_VERTICALRESPONSE_MOBILE_PHONE'),
			'fax'                 => JText::_('RSFP_VERTICALRESPONSE_FAX'),
			'website'             => JText::_('RSFP_VERTICALRESPONSE_WEBSITE'),
			'title'               => JText::_('RSFP_VERTICALRESPONSE_TITLE'),
			'company'        	  => JText::_('RSFP_VERTICALRESPONSE_COMPANY_NAME'),
			'street_address'      => JText::_('RSFP_VERTICALRESPONSE_ADDRESS1'),
			'extended_address'    => JText::_('RSFP_VERTICALRESPONSE_ADDRESS2'),
			'locality'            => JText::_('RSFP_VERTICALRESPONSE_LOCALITY'),
			'postal_code'         => JText::_('RSFP_VERTICALRESPONSE_POSTAL_CODE'),
			'region'              => JText::_('RSFP_VERTICALRESPONSE_REGION'),
			'country_code_alpha2' => JText::_('RSFP_VERTICALRESPONSE_COUNTRYCODE')
		);
	}

	/**
	 * Function returns the USER ID if it's present in the contact list,
	 * else - returns false
	 *
	 * @param $args
	 *
	 * @return bool
	 * @throws Exception
	 */
	public function userExists($args)
	{
		$client = new VR_APIClient(null);
		$search   = array(
			'type'  => 'all',
			'email' => $args['email']
		);

		$response = $client->get(ROOT_URL . 'contacts', $search);

		if ($response == 'Invalid or expired token')
		{
			throw new Exception(JText::_('RSFP_VERTICALRESPONSE_INVALIDTOKEN'));
		}

		if (empty($response['items']))
		{
			return false;
		}

		return $response['items'][0]['attributes']['id'];
	}

	/**
	 * @param $formId
	 */
	public function onRsformFormDelete($formId)
	{
		if ($row = $this->getTable())
		{
			$row->delete($formId);
		}
	}

	/**
	 * @param $form
	 * @param $xml
	 * @param $fields
	 */
	public function onRsformFormBackup($form, $xml, $fields)
	{
		if ($row = $this->getTable())
		{
			if ($row->load($form->FormId))
			{
				$row->check();

				$data = $row->getProperties();
				unset($data['form_id']);

				$xml->add('verticalresponse');
				foreach ($data as $property => $value)
				{
					$xml->add($property, $value);
				}
				$xml->add('/verticalresponse');
			}
		}
	}

	/**
	 * @param $form
	 * @param $xml
	 * @param $fields
	 */
	public function onRsformFormRestore($form, $xml, $fields)
	{
		if (isset($xml->verticalresponse))
		{
			$data = array(
				'form_id' => $form->FormId
			);
			foreach ($xml->verticalresponse->children() as $property => $value)
			{
				$data[$property] = (string) $value;
			}
			$row = $this->getTable();
			$row->save($data);
		}
	}

	/**
	 *
	 */
	public function onRsformBackendFormRestoreTruncate()
	{
		JFactory::getDbo()->truncateTable('#__rsform_verticalresponse');
	}
}