<?php
/**
* @package RSForm!Pro
* @copyright (C) 2007-2019 www.rsjoomla.com
* @license GPL, http://www.gnu.org/copyleft/gpl.html
*/

// no direct access
defined('_JEXEC') or die('Restricted access');

class plgSystemRsfpsalesforceInstallerScript
{
	protected static $minJoomla = '3.7.0';
	protected static $minComponent = '3.0.0';

	public function preflight($type, $parent)
	{
		if ($type == 'uninstall')
		{
			return true;
		}

		try
		{
			$jversion = new JVersion();
			if (!$jversion->isCompatible(static::$minJoomla))
			{
				throw new Exception(sprintf('Please upgrade to at least Joomla! %s before continuing!', static::$minJoomla));
			}

			if (!file_exists(JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/rsform.php'))
			{
				throw new Exception('Please install the RSForm! Pro component before continuing.');
			}

			if (!file_exists(JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/assets.php') || !file_exists(JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/version.php'))
			{
				throw new Exception(sprintf('Please upgrade RSForm! Pro to at least version %s before continuing!', static::$minComponent));
			}

			// Check version matches
			require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/version.php';

			if (!class_exists('RSFormProVersion') || version_compare((string) new RSFormProVersion, static::$minComponent, '<'))
			{
				throw new Exception(sprintf('Please upgrade RSForm! Pro to at least version %s before continuing!', static::$minComponent));
			}
		}
		catch (Exception $e)
		{
			JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
			return false;
		}

		return true;
	}
	
	public function update($parent)
	{
		$db = JFactory::getDbo();

		$this->runSQL($parent->getParent()->getPath('source'), 'install');
		$this->copyFiles($parent);

		$columns = $db->getTableColumns('#__rsform_salesforce');
		if (!isset($columns['slsf_custom_fields']))
		{
			$db->setQuery("ALTER TABLE `#__rsform_salesforce` ADD `slsf_industry` VARCHAR( 255 ) NOT NULL AFTER `slsf_debugEmail` ,".
						  "ADD `slsf_description` TEXT NOT NULL AFTER `slsf_industry` ,".
						  "ADD `slsf_mobile` VARCHAR( 255 ) NOT NULL AFTER `slsf_description` ,".
						  "ADD `slsf_fax` VARCHAR( 255 ) NOT NULL AFTER `slsf_mobile` ,".
						  "ADD `slsf_website` VARCHAR( 255 ) NOT NULL AFTER `slsf_fax` ,".
						  "ADD `slsf_salutation` VARCHAR( 255 ) NOT NULL AFTER `slsf_website` ,".
						  "ADD `slsf_revenue` VARCHAR( 255 ) NOT NULL AFTER `slsf_salutation` ,".
						  "ADD `slsf_employees` VARCHAR( 255 ) NOT NULL AFTER `slsf_revenue` ,".
						  "ADD `slsf_custom_fields` TEXT NOT NULL AFTER `slsf_employees`");
			$db->execute();
		}

		if (!isset($columns['slsf_campaign_id']))
		{
			$db->setQuery("ALTER TABLE `#__rsform_salesforce` ADD `slsf_campaign_id` VARCHAR( 255 ) NOT NULL AFTER `slsf_custom_fields`");
			$db->execute();
		}

		if (!isset($columns['slsf_donotcall']))
		{
			$db->setQuery("ALTER TABLE `#__rsform_salesforce` ADD `slsf_donotcall` VARCHAR( 255 ) NOT NULL AFTER `slsf_campaign_id`, ADD `slsf_emailoptout` VARCHAR( 255 ) NOT NULL AFTER `slsf_donotcall`,
		ADD `slsf_faxoptout` VARCHAR( 255 ) NOT NULL AFTER `slsf_emailoptout`");
			$db->execute();
		}

		if (!isset($columns['slsf_country']))
		{
			$db->setQuery("ALTER TABLE `#__rsform_salesforce` ADD `slsf_country` VARCHAR( 255 ) NOT NULL AFTER `slsf_zip`");
			$db->execute();
		}

		if (!isset($columns['slsf_action']))
		{
			$db->setQuery("ALTER TABLE `#__rsform_salesforce` ADD `slsf_action` TINYINT(1) NOT NULL DEFAULT '1' AFTER `form_id`")->execute();
		}
		if (!isset($columns['slsf_action_field']))
		{
			$db->setQuery("ALTER TABLE `#__rsform_salesforce` ADD `slsf_action_field` TEXT AFTER `slsf_action`")->execute();
		}
		if (!isset($columns['slsf_recordtype']))
		{
			$db->setQuery("ALTER TABLE `#__rsform_salesforce` ADD `slsf_recordtype` VARCHAR( 255 ) DEFAULT '' AFTER `slsf_campaign_id`")->execute();
		}
	}
	
	public function install($parent)
	{
		$this->copyFiles($parent);
	}
	
	protected function copyFiles($parent)
	{
		$app = JFactory::getApplication();
		$installer = $parent->getParent();
		$src = $installer->getPath('source').'/admin';
		$dest = JPATH_ADMINISTRATOR.'/components/com_rsform';
		
		if (!JFolder::copy($src, $dest, '', true)) {
			$app->enqueueMessage('Could not copy to '.str_replace(JPATH_SITE, '', $dest).', please make sure destination is writable!', 'error');
		}
	}

	/**
	 * @param $source
	 * @param $file
	 *
	 * @throws Exception
	 */
	protected function runSQL($source, $file)
	{
		$db = JFactory::getDbo();
		$sqlfile = $source . '/sql/mysql/' . $file . '.sql';

		if (file_exists($sqlfile))
		{
			$buffer = file_get_contents($sqlfile);
			if ($buffer !== false)
			{
				$queries = $db->splitSql($buffer);
				foreach ($queries as $query)
				{
					$query = trim($query);
					if ($query != '')
					{
						$db->setQuery($query)->execute();
					}
				}
			}
		}
	}
}